@@ -22,32 +22,40 @@
 %%% @end
 -module(e3d_transform).
 
--export([%% Initilizes matrices
+-export([%% Initializes matrices
 	 identity/0, init/1,
-	 lookat/3, ortho/2, ortho/6, perspective/3, perspective/4, pick/5,
+	 lookat/3, ortho/2, ortho/6, perspective/3, perspective/4, pick/5, frustum/6,
 	 %% Get the actual matrices
 	 matrix/1, inv_matrix/1,
 	 %% Transform the matrices
-	 inverse/1, translate/2, rotate/2, rotate/3, scale/2, mul/1, mul/2
+	 inverse/1, translate/2, rotate/2, rotate/3, scale/2, mul/1, mul/2,
+         %% (un)Projection
+         project/4, unproject/4
 	]).
 
-
 -include("e3d.hrl").
 
+-type transform() :: #e3d_transf{}.
+
+-type matrix() :: e3d_mat:matrix().
+-type vector() :: e3d_vec:vector().
+-type point() :: e3d_vec:point().
+
+
 %%%-------------------------------------------------------------------
 %%--------------------------------------------------------------------
 %% @doc  Returns the identity transform
 %% @end
 %%--------------------------------------------------------------------
--spec identity() -> e3d_transform().
+-spec identity() -> transform().
 identity() ->
     #e3d_transf{}.
 
 %%--------------------------------------------------------------------
-%% @doc  Initilizes transform from matrix mat
+%% @doc  Initializes transform from matrix mat
 %% @end
 %%--------------------------------------------------------------------
--spec init(e3d_matrix()) -> e3d_transform().
+-spec init(matrix()) -> transform().
 init(Mat) when tuple_size(Mat) =:= 12 ->
     init(e3d_mat:expand(Mat));
 init(Mat) ->
@@ -57,14 +65,14 @@ init(Mat) ->
 %% @doc  Returns the matrix
 %% @end
 %%--------------------------------------------------------------------
--spec matrix(e3d_transform()) -> e3d_matrix().
+-spec matrix(transform()) -> matrix().
 matrix(#e3d_transf{mat=M}) -> e3d_mat:expand(M).
 
 %%--------------------------------------------------------------------
 %% @doc  Returns the inverse matrix
 %% @end
 %%--------------------------------------------------------------------
--spec inv_matrix(e3d_transform()) -> e3d_matrix().
+-spec inv_matrix(transform()) -> matrix().
 inv_matrix(#e3d_transf{inv=I}) -> e3d_mat:expand(I).
 
 %%%-------------------------------------------------------------------
@@ -74,7 +82,7 @@ inv_matrix(#e3d_transf{inv=I}) -> e3d_mat:expand(I).
 %% @doc  Inverses the transform
 %% @end
 %%--------------------------------------------------------------------
--spec inverse(e3d_transform()) -> e3d_transform().
+-spec inverse(transform()) -> transform().
 inverse(#e3d_transf{mat=M, inv=I}) ->
     #e3d_transf{mat=I, inv=M}.
 
@@ -82,7 +90,7 @@ inverse(#e3d_transf{mat=M, inv=I}) ->
 %% @doc  Translates the matrix with vector
 %% @end
 %%--------------------------------------------------------------------
--spec translate(e3d_transform(), e3d_vector()) -> e3d_transform().
+-spec translate(transform(), vector()) -> transform().
 translate(#e3d_transf{mat=M,inv=I}, {Dx,Dy,Dz}) ->
     #e3d_transf{mat = e3d_mat:mul(M, e3d_mat:translate(Dx,Dy,Dz)),
 		inv = e3d_mat:mul(e3d_mat:translate(-Dx,-Dy,-Dz), I)}.
@@ -91,17 +99,17 @@ translate(#e3d_transf{mat=M,inv=I}, {Dx,Dy,Dz}) ->
 %% @doc  Rotates the matrix with rotation matrix
 %% @end
 %%--------------------------------------------------------------------
--spec rotate(e3d_transform(), e3d_matrix()) -> e3d_transform().
+-spec rotate(transform(), matrix()) -> transform().
 rotate(#e3d_transf{mat=M,inv=I}, Rot)
   when tuple_size(Rot) =:= 12; tuple_size(Rot) =:= 16 ->
     #e3d_transf{mat = e3d_mat:mul(M, Rot),
 		inv = e3d_mat:mul(e3d_mat:transpose(Rot), I)}.
 
 %%--------------------------------------------------------------------
-%% @doc  Rotates the matrix with angle (in degress) and direction
+%% @doc  Rotates the matrix with angle (in degrees) and direction
 %% @end
 %%--------------------------------------------------------------------
--spec rotate(e3d_transform(), number(), e3d_vector()) -> e3d_transform().
+-spec rotate(transform(), number(), vector()) -> transform().
 rotate(Mat = #e3d_transf{}, A, Vec) ->
     rotate(Mat, e3d_mat:rotate(A,Vec)).
 
@@ -109,7 +117,7 @@ rotate(Mat = #e3d_transf{}, A, Vec) ->
 %% @doc  Scales the matrix with {ScaleX, ScaleY, ScaleZ}
 %% @end
 %%--------------------------------------------------------------------
--spec scale(e3d_transform(), e3d_vector()) -> e3d_transform().
+-spec scale(transform(), vector()) -> transform().
 scale(#e3d_transf{mat=M,inv=I}, {X,Y,Z}) ->
     #e3d_transf{mat = e3d_mat:mul(M, e3d_mat:scale(X,Y,Z)),
 		inv = e3d_mat:mul(e3d_mat:scale(1/X,1/Y,1/Z), I)}.
@@ -119,24 +127,57 @@ scale(#e3d_transf{mat=M,inv=I}, {X,Y,Z}) ->
 %%       Trans(Vec) = Mat(Current(Vec))
 %% @end
 %%----------------------------------------------------------------------
--spec mul(e3d_transform(), e3d_transform()) -> e3d_transform().
+-spec mul(transform(), transform()) -> transform().
 mul(#e3d_transf{mat=M1,inv=I1}, #e3d_transf{mat=M2,inv=I2}) ->
     #e3d_transf{mat = e3d_mat:mul(M1, M2), inv = e3d_mat:mul(I2, I1)}.
 
 %%--------------------------------------------------------------
 %% mul([Rx,Ry,Rz]) = mul([mul(Ry,Rx),Rz])
 %%--------------------------------------------------------------
--spec mul([e3d_transform()]) -> e3d_transform().
+-spec mul([transform()]) -> transform().
 mul([#e3d_transf{}=A,#e3d_transf{}=B | T ]) -> mul([mul(B,A) | T]);
 mul([#e3d_transf{}=A]) -> A.
 
+%%----------------------------------------------------------------------
+%% @doc  Transforms point to window coordinates
+%%
+%% @end
+%%----------------------------------------------------------------------
+-spec project(Point::point(),
+              ModelView::transform(),
+              Projection::transform(), ViewPort::{integer(), integer(), integer(), integer()}) ->
+          point().
+project(Point, #e3d_transf{mat=MV}, #e3d_transf{mat=Pr}, {V0,V1,V2,V3}) ->
+    P0 = e3d_mat:mul_point(MV, Point),
+    P1 = e3d_mat:mul_point(Pr, P0),
+    {Px,Py,Pz} = e3d_vec:add_prod({0.5,0.5,0.5}, P1, 0.5),
+    {Px*V2+V0, Py*V3+V1, Pz}.
+
+%%----------------------------------------------------------------------
+%% @doc  Maps windows coordinates to object coordinates
+%%
+%% @end
+%%----------------------------------------------------------------------
+-spec unproject(WinPoint::e3d_vec:point(),
+                ModelView::transform(),
+                Projection::transform(), ViewPort::{integer(), integer(), integer(), integer()}) ->
+          e3d_vec:point().
+unproject({WinX,WinY,WinZ}, #e3d_transf{inv=InvMV}, #e3d_transf{inv=InvPr}, {V0,V1,V2,V3}) ->
+    %% From windows coords to 0.0-1.0
+    Point0 = {(WinX-V0)/V2, (WinY-V1)/V3, WinZ},
+    %% From 0.0-1.0 -> -1, 1
+    Point = e3d_vec:add_prod({-1.0,-1.0,-1.0}, Point0, 2.0),
+    %% ModelView Projection matrix
+    MVPM = e3d_mat:mul(InvMV,InvPr),
+    e3d_mat:mul_point(MVPM, Point).
+
 %%%-------------------------------------------------------------------
 
 %%--------------------------------------------------------------------
 %% @doc  Generates a world to camera transformation
 %% @end
 %%--------------------------------------------------------------------
--spec lookat(e3d_point(), e3d_vector(), e3d_vector()) -> e3d_transform().
+-spec lookat(point(), vector(), vector()) -> transform().
 lookat(Pos, Look, Up) ->
     Dir = e3d_vec:norm_sub(Look, Pos),
     Right = e3d_vec:norm(e3d_vec:cross(Dir, e3d_vec:norm(Up))),
@@ -153,10 +194,11 @@ lookat(Pos, Look, Up) ->
 %% @doc  Generates a ortho transformation
 %% @end
 %%--------------------------------------------------------------------
--spec ortho(float(), float()) -> e3d_transform().
+-spec ortho(float(), float()) -> transform().
 ortho(Near, Far) ->
-    ortho(-1.0, 1.0, -1.0, 1.0, Far, Near).
+    ortho(-1.0, 1.0, -1.0, 1.0, Near, Far).
 
+-spec ortho(float(), float(), float(), float(), float(), float()) -> transform().
 ortho(Left, Right, Bottom, Top, Near, Far) ->
     O = 0.0,
     IDx = 1/(Right-Left),
@@ -169,13 +211,10 @@ ortho(Left, Right, Bottom, Top, Near, Far) ->
 	    O,   O, O},
     %% Do this in 3 steps to avoid inverse calculation problems
     Mat1 = scale(init(Mat0), {IDx,IDy,IDz}),
-    Trans = translate(identity(), {-(Right+Left)*IDx, 
-				   -(Top+Bottom)*IDy, 
-				   -(Far+Near)*IDz}),
+    Tv = {-(Right+Left)*IDx, -(Top+Bottom)*IDy, -(Far+Near)*IDz},
+    Trans = translate(identity(), Tv),
     mul(Trans, Mat1).
 
-    
-
 %%--------------------------------------------------------------------
 %% @doc  Generates a perspective transformation
 %%       Fov = Field Of View (in degrees)
@@ -184,12 +223,12 @@ ortho(Left, Right, Bottom, Top, Near, Far) ->
 %% @end
 %%--------------------------------------------------------------------
 -spec perspective(Fov::float(),
-		  Near::float(), Far::float()) -> e3d_transform().
+		  Near::float(), Far::float()) -> transform().
 perspective(Fov, Near, Far) ->
     perspective(Fov, 1.0, Near, Far).
 
 -spec perspective(Fov::float(), Aspect::float(),
-		  Near::float(), Far::float()) -> e3d_transform().
+		  Near::float(), Far::float()) -> transform().
 perspective(Fov, Aspect, Near, Far) ->
     T = 1.0 / math:tan((Fov*math:pi()/180)/2.0),
     %% Perform projective divide
@@ -202,6 +241,33 @@ perspective(Fov, Aspect, Near, Far) ->
     InvPersp = e3d_mat:invert(Persp),
     scale(#e3d_transf{mat=Persp, inv=InvPersp}, {T,T,1.0}).
 
+%%--------------------------------------------------------------------
+%% @doc  Generates a perspective projection matrix
+%% @end
+%%--------------------------------------------------------------------
+
+-spec frustum(Left::float(), Right::float(),
+              Bottom::float(), Top::float(),
+              Near::float(), Far::float()) ->
+          transform().
+frustum(Left, Right, Bottom, Top, Near, Far) ->
+    InvX = 1/(Right-Left),
+    InvY = 1/(Top-Bottom),
+    InvZ = 1/(Far-Near),
+    A = (Right+Left)*InvX,
+    B = (Top+Bottom)*InvY,
+    C = -(Far+Near)*InvZ,
+    D = -(2*Far*Near)*InvZ,
+    I = 1.0, O = 0.0,
+    E = 2*Near*InvX,
+    F = 2*Near*InvY,
+
+    Persp = {E,   O,   O,  O,
+	     O,   F,   O,  O,
+	     A,   B,   C, -I,
+	     O,   O,   D,  O},
+    InvPersp = e3d_mat:invert(Persp),
+    #e3d_transf{mat=Persp, inv=InvPersp}.
 
 %%--------------------------------------------------------------------
 %% @doc  Generates a pick matrix transformation
@@ -211,7 +277,7 @@ perspective(Fov, Aspect, Near, Far) ->
 -spec pick(X::float(), Y::float(),
 	   Width::float(), Height::float(),
 	   Viewport::{integer(),integer(),integer(),integer()}
-	  ) -> e3d_transform().
+	  ) -> transform().
 pick(X, Y, W, H, {X0,Y0,X1,Y1}) ->
     Sx = X1 / W,
     Sy = Y1 / H,
