@@ -60,17 +60,26 @@ auv_menu(1,_What) -> {?MODULE, segment};
 auv_menu(2,_) -> {?MODULE, segment_old};
 auv_menu(3,_) -> {?MODULE, force_seg}.
 
+auv_show_menu(label) ->
+    ?__(1,"Show/Hide Background Image");
+auv_show_menu(help) ->
+    ?__(2,"Toggle display of the background texture image");
 auv_show_menu(Action) ->
     Cmd = {show,toggle_background},
     case Action of
 	true ->
-	    Label = ?__(1,"Show/Hide Background Image"),
-	    Help = ?__(2,"Toggle display of the background texture image"),
+	    Label = auv_show_menu(label),
+	    Help = auv_show_menu(help),
 	    wings_menu:update_menu(view, Cmd, {append, 0, Label},Help);
 	false ->
 	    wings_menu:update_menu(view, Cmd, delete)
     end.
 
+auv_export_menu(label) ->
+    ?__(1,"Export UV...");
+auv_export_menu(help) ->
+    ?__(2,"Exports the UV as cartoon edges (.eps, .svg)").
+
 command({body,{?MODULE, Op}} , St) ->
     start_uvmap(Op, St);
 command({face,{?MODULE, Op}} , St) ->
@@ -140,8 +149,7 @@ create_window(Action, Name, Id, #st{shapes=Shs}=St) ->
 
     {Pos,Size} = init_drawarea(),
     {Frame,Ps} = wings_frame:make_win(Title, [{size, Size}, {pos, Pos}]),
-    Context = wxGLCanvas:getContext(?GET(gl_canvas)),
-    Canvas = wings_gl:window(Frame, Context, true, true),
+    Canvas = wings_gl:window(Frame, ?GET(gl_context), true, true),
     Props = [{display_data,Name}|wings_view:initial_properties()++Ps],
     wings_wm:toplevel(Name, Canvas, Props, Op),
     wings_wm:send(Name, {init,{Action,We}}),
@@ -259,9 +267,9 @@ create_uv_state(Charts, MatName, Fs, We, #st{shapes=Shs0}=GeomSt) ->
     wings_wm:later(got_focus),
 
     Win = wings_wm:this(),
-    case get({?MODULE,show_background}) of
+    case ?GET({?MODULE,show_background}) of
 	undefined -> 
-	    put({?MODULE,show_background}, true);
+	    ?SET({?MODULE,show_background}, true);
 	_ -> ignore
     end,
     wings:register_postdraw_hook(Win, ?MODULE,
@@ -365,14 +373,32 @@ bg_img_id() ->
 %%%% Menus.
 
 command_menu(body, X, Y) ->
+    First = [{?__(37,"Stretch optimization"), stretch_opt,
+              ?__(38,"Optimize the chart stretch")},
+             separator],
+    Unfold = case erlang:system_info(wordsize) of
+                 4 ->
+                     [{?__(39,"Unfold"), lsqcm, ?__(40,"Unfold the chart")}];
+                 8 ->
+                     [{?__(39,"Unfold"), lsqcm, ?__(40,"Unfold the chart")},
+                      {?__(391,"Unfold (slow)"),slim,
+                       ?__(392,"Unfold the chart, slow for charts with many faces but better")}]
+             end,
+    Rest = [{?__(41,"Project Normal"), project,
+             ?__(42,"Project UVs from chart normal")},
+            {?__(43,"Spherical"), sphere,
+             ?__(44,"Spherical mapping")}
+           ],
+    Remap = First ++ Unfold ++ Rest,
+
     Menu = [{?__(2,"Move"), {move, move_directions(false)},
 	     ?__(3,"Move selected charts")},
 	    {?__(4,"Scale"), {scale, scale_directions(false) ++
-			      [separator] ++ stretch_directions() ++
-			      [separator,
-			       {?__(411,"Normalize Sizes"), normalize,
-				?__(412,"Normalize Chart Sizes so that each"
-				    "chart get it's corresponding 2d area")}]},
+                                  [separator] ++ stretch_directions() ++
+                                  [separator,
+                                   {?__(411,"Normalize Sizes"), normalize,
+                                    ?__(412,"Normalize Chart Sizes so that each"
+                                        "chart get it's corresponding 2d area")}]},
 	     ?__(5,"Scale selected charts")},
 	    {?__(6,"Rotate"), rotate, ?__(7,"Rotate selected charts")},
 	    separator,
@@ -386,9 +412,16 @@ command_menu(body, X, Y) ->
 	       {?__(19,"Left"), left, ?__(20,"Move to left border")},
 	       {?__(21,"Right"), right, ?__(22,"Move to right border")}
 	      ]}, ?__(23,"Move charts to position")},
+	    {?__(93,"Align"),
+	     {align,
+	      [{?__(15,"Bottom"), bottom, ?__(95,"Align to bottom")},
+	       {?__(17,"Top"), top, ?__(96,"Align to top")},
+	       {?__(19,"Left"), left, ?__(97,"Align to left")},
+	       {?__(21,"Right"), right, ?__(98,"Align to right")}
+	      ]}, ?__(94,"Align charts relative each other")},
 	    {?__(24,"Flip"),{flip,
-		     [{?__(25,"Horizontal"),horizontal,?__(26,"Flip selection horizontally")},
-		      {?__(27,"Vertical"),vertical,?__(28,"Flip selection vertically")}]},
+                             [{?__(25,"Horizontal"),horizontal,?__(26,"Flip selection horizontally")},
+                              {?__(27,"Vertical"),vertical,?__(28,"Flip selection vertically")}]},
 	     ?__(29,"Flip selected charts")},
 	    separator,
 	    {?__(30,"Tighten"),tighten,
@@ -397,15 +430,7 @@ command_menu(body, X, Y) ->
 	    {?__(32,"Hide"),hide,?__(33,"Hide selected charts but keep UV-coordinates")},
 	    {?__(34,"Delete"),delete,?__(35,"Remove UV-coordinates for the selected charts")},
 	    separator,
-	    {?__(36,"ReMap UV"), {remap, [{?__(37,"Stretch optimization"), stretch_opt, 
-				   ?__(38,"Optimize the chart stretch")},
-				  separator,
-				  {?__(39,"Unfold"), lsqcm, ?__(40,"Unfold the chart")},
-				  {?__(41,"Project Normal"), project, 
-				   ?__(42,"Project UVs from chart normal")},
-				  {?__(43,"Spherical"), sphere, 
-				   ?__(44,"Spherical mapping")}
-				 ]}, 
+	    {?__(36,"ReMap UV"), {remap, Remap},
 	     ?__(45,"Calculate new UVs with chosen algorithm")}
 	   ] ++ option_menu(),
     wings_menu:popup_menu(X,Y, {auv,body}, Menu);
@@ -416,7 +441,7 @@ command_menu(face, X, Y) ->
 	    {?__(49,"Scale"),{scale,Scale},?__(50,"Scale selected faces"), [magnet]},
 	    {?__(51,"Rotate"),rotate,?__(52,"Rotate selected faces"), [magnet]},
 	    separator,
-	    {?__(521,"Project-Unfold"),	proj_lsqcm,
+	    {?__(521,"Project-Unfold"),	{remap, proj_lsqcm},
 	     ?__(522,"Project selected faces from normal and unfold the rest of chart")}
 	   ] ++ option_menu(),
     wings_menu:popup_menu(X,Y, {auv,face}, Menu);
@@ -444,11 +469,11 @@ command_menu(edge, X, Y) ->
 command_menu(vertex, X, Y) ->
     Scale = scale_directions(true),
     Move = move_directions(true),
-    Align = 	    
+    Align =
 	[{?__(70,"Free"),free,?__(71,"Rotate selection freely"), [magnet]},
-	 {?__(72,"Chart to X"), align_x, 
+	 {?__(72,"Chart to X"), align_x,
 	  ?__(73,"Rotate chart to align (imaginary) edge joining selected verts to X-axis")},
-	 {?__(74,"Chart to Y"), align_y, 
+	 {?__(74,"Chart to Y"), align_y,
 	  ?__(75,"Rotate chart to align (imaginary) edge joining selected verts to Y-axis")}],
 
     Menu = [{?__(77,"Move"),{move,Move},?__(78,"Move selected vertices"),[magnet]},
@@ -456,24 +481,33 @@ command_menu(vertex, X, Y) ->
 	    {?__(81,"Rotate"),{rotate,Align},?__(82,"Rotation commands")},
 	    separator,
 	    {?__(83,"Flatten"),{flatten,
-			[{"X", x, ?__(84,"Flatten horizontally")},
-			 {"Y", y, ?__(85,"Flatten vertically")}]}, 
+                                [{"X", x, ?__(84,"Flatten horizontally")},
+                                 {"Y", y, ?__(85,"Flatten vertically")}]},
 	     ?__(86,"Flatten selected vertices")},
 	    {?__(87,"Tighten"),tighten,
 	     ?__(88,"Move UV coordinates towards average midpoint"),
 	     [magnet]},
-	    separator, 
-	    {?__(89,"Unfold"),lsqcm,?__(90,"Unfold the chart (without moving the selected vertices)")},
+	    separator,
+	    {?__(89,"Unfold"),{remap, lsqcm},?__(90,"Unfold the chart (without moving the selected vertices)")},
 	    {?__(91,"SphereMap"),sphere,?__(92,"Create a spherical mapping with "
 	     "selected vertices being North/South pole")}
 	   ] ++ option_menu(),
     wings_menu:popup_menu(X,Y, {auv,vertex}, Menu);
 command_menu(_, X, Y) ->
-    [_|Menu] = option_menu(),
+    case catch wpc_hlines:init() of
+        true -> ExportMenu = [separator, {auv_export_menu(label), export_uv, auv_export_menu(help)}];
+        _ -> ExportMenu = []
+    end,
+    Checked = [{crossmark, ?GET({?MODULE,show_background})}],
+    Menu = [{auv_show_menu(label),toggle_background,auv_show_menu(help),Checked}] ++
+           ExportMenu ++ option_menu(),
     wings_menu:popup_menu(X,Y, {auv,option}, Menu).
 
 stretch_directions() ->
-    [{?__(1,"Max Uniform"), max_uniform, ?__(2,"Maximize either horizontally or vertically")},
+    [{?__(1,"Max Uniform"), max_uniform(),
+      {?__(2,"Maximize either horizontally or vertically"),
+       ?__(7,"Maximize by using the horizontal dimension"),
+       ?__(8,"Maximize by using the vertical dimension")}, []},
      {?__(3,"Max Horizontal"), max_x, ?__(4,"Maximize horizontally (X dir)")},
      {?__(5,"Max Vertical"),   max_y, ?__(6,"Maximize vertically (Y dir)")}].
 
@@ -491,11 +525,18 @@ scale_directions(true) ->
      {?__(3,"Horizontal"), x, ?__(4,"Scale horizontally (X dir)"), [magnet]},
      {?__(5,"Vertical"),   y, ?__(6,"Scale vertically (Y dir)"), [magnet]}];
 scale_directions(false) ->
-    [{?__(1,"Uniform"),    uniform, ?__(2,"Scale in both directions")},
+    [{?__(1,"Uniform"),    uniform, ?__(2,"Scale in both directions"), []},
      {?__(3,"Horizontal"), x, ?__(4,"Scale horizontally (X dir)")},
      {?__(5,"Vertical"),   y, ?__(6,"Scale vertically (Y dir)")}].
 
 
+max_uniform() ->
+    fun
+        (1, _Ns) -> {auv,{scale,max_uniform}};
+        (2, _Ns) -> {auv,{scale,{max_uniform,x}}};
+        (3, _Ns) -> {auv,{scale,{max_uniform,y}}}
+    end.
+
 option_menu() ->
     [separator,
      {?__(1,"Create Texture"),create_texture,?__(2,"Make and Attach a texture to the model")}].
@@ -612,17 +653,19 @@ handle_event_3(#mousebutton{button=?SDL_BUTTON_RIGHT}=Ev,
     end;
 handle_event_3({drop,DropData}, St) ->
     handle_drop(DropData, St);
-handle_event_3({action,{{auv,_},create_texture}},_St) ->
-    auv_texture:draw_options();
-handle_event_3({action,{auv,{draw_options,restart}}}, _St) ->
-    auv_texture:draw_options();
+handle_event_3({action,{{auv,_},create_texture}}, St) ->
+    ?SET({?MODULE,show_background}, true),
+    auv_texture:draw_options(St);
+handle_event_3({action,{auv,{draw_options,restart}}}, St) ->
+    ?SET({?MODULE,show_background}, true),
+    auv_texture:draw_options(St);
 handle_event_3({action,{auv,{draw_options,Opt}}}, #st{bb=Uvs}=St) ->
     #uvstate{st=GeomSt0,matname=MatName0,bg_img=Image} = Uvs,
     Tx = ?SLOW(auv_texture:get_texture(St, Opt)),
-    case MatName0 of 
-	none -> 
+    case MatName0 of
+	none ->
 	    ok = wings_image:update(Image, Tx),
-	    put({?MODULE,show_background}, true),
+	    ?SET({?MODULE,show_background}, true),
 	    get_event(St);
 	_ ->
 	    TexName = case get_texture(MatName0, St) of
@@ -716,9 +759,7 @@ handle_event_3({action,Ev}=Act, #st{selmode=AUVSel, bb=#uvstate{st=#st{selmode=G
 	{_, circularise} ->
 	    handle_command(circularise,St);
 	{view,{show,toggle_background}} ->
-	    Old = get({?MODULE,show_background}),
-	    put({?MODULE,show_background},not Old),
-	    wings_wm:dirty();
+	    handle_command(toggle_background,St);
 	{view,aim} ->
 	    St1 = fake_selection(St),
 	    wings_view:command(aim, St1),
@@ -820,15 +861,6 @@ handle_command_1({'ASK',Ask}, St) ->
 handle_command_1({remap,Method}, St0) ->
     St = remap(Method, St0),
     get_event(St);
-handle_command_1(M=proj_lsqcm, St0) ->
-    St = remap(M,St0),
-    get_event(St);
-handle_command_1(lsqcm, St0) ->
-    St = reunfold(lsqcm,St0),
-    get_event(St);
-handle_command_1(sphere, St0) ->
-    St = reunfold(sphere,St0),
-    get_event(St);
 handle_command_1(move, St) ->
     wings_move:setup(free_2d, St);
 handle_command_1({move,{'ASK',Ask}}, St) ->
@@ -838,7 +870,8 @@ handle_command_1({move,{'ASK',Ask}}, St) ->
 handle_command_1({move,Axis}, St) ->
     wings_move:setup(Axis, St);
 handle_command_1({scale,Dir}, St0) %% Maximize chart
-  when Dir == max_uniform; Dir == max_x; Dir == max_y -> 
+  when Dir == max_uniform; Dir == {max_uniform,x}; Dir == {max_uniform,y};
+       Dir == max_x; Dir == max_y ->
     St1 = wpa:sel_map(fun(_, We) -> stretch(Dir,We) end, St0),
     St = update_selected_uvcoords(St1),
     get_event(St);
@@ -893,6 +926,18 @@ handle_command_1({move_to,Dir}, St0) ->
     St1 = wpa:sel_map(fun(_, We) -> move_to(Dir,We) end, St0),
     St = update_selected_uvcoords(St1),
     get_event(St);
+handle_command_1({align,Dir}, #st{selmode=Mode,sel=[{_,Els}]}=St0) ->
+    St =
+        case gb_sets:size(Els) of
+            1 ->
+                align_error(Mode),
+                St0;
+            _ ->
+                BB = wings_sel:bounding_box(St0),
+                St1 = wpa:sel_map(fun(_, We) -> align(Dir,BB,We) end, St0),
+                update_selected_uvcoords(St1)
+        end,
+    get_event(St);
 handle_command_1({flip,horizontal}, St0) ->
     St1 = wpa:sel_map(fun(_, We) -> flip_horizontal(We) end, St0),
     St = update_selected_uvcoords(St1),
@@ -952,7 +997,12 @@ handle_command_1(cut_edges, St0 = #st{selmode=edge,bb=#uvstate{id=Id,st=Geom}})
     St2 = displace_cuts(Es, St1),
     St  = update_selected_uvcoords(St2),
     get_event(St);
-
+handle_command_1(toggle_background, _) ->
+    Old = ?GET({?MODULE,show_background}),
+    ?SET({?MODULE,show_background},not Old),
+    wings_wm:dirty();
+handle_command_1(export_uv, #st{}=St) ->
+    wpc_hlines:command({file, {export_uv, {eps, true}}}, St);
 handle_command_1(Cmd, #st{selmode=Mode}=St0) ->
     case wings_plugin:command({{auv,Mode},Cmd}, St0) of
       next ->
@@ -1003,16 +1053,16 @@ replace_ask(Tuple0, AskArgs) when is_tuple(Tuple0) ->
     list_to_tuple(Tuple);
 replace_ask(Term, _) -> Term.
 
-repeatable({remap,_},Mode) -> Mode == body;
-repeatable(lsqcm, Mode) -> Mode == vertex;
-repeatable(sphere, Mode)-> Mode == vertex;
-repeatable(proj_lsqcm, Mode) -> Mode == face;
+repeatable({remap, proj_lsqcm}, Mode) -> Mode == face;
+repeatable({remap, proj_slim}, Mode) -> Mode == face;
+repeatable({remap,_},body) -> true;
+repeatable({remap,_},Mode) -> Mode == vertex;
 repeatable({scale, Dir}, Mode) 
   when ((Dir == max_uniform) or (Dir == max_x) or (Dir == max_y) or 
 	(Dir == normalize)) and (Mode /= body) -> false;
 repeatable({rotate, Dir}, Mode) 
   when ((Dir == align_y) or (Dir == align_x) or (Dir == align_xy)) 
-       and ((Mode == body) or (Mode == face)) -> no;
+       and ((Mode == body) or (Mode == face)) -> false;
 repeatable({move_to,_},Mode) -> Mode == body;
 repeatable({flip,_},Mode) -> Mode == body;
 repeatable(slide, Mode) -> Mode == edge;
@@ -1709,7 +1759,7 @@ align_chart(Dir, St = #st{selmode=Mode}) ->
 		      align_chart(Dir,array:get(V1,Vtab),
 				  array:get(V2,Vtab),
 				  We);
-		  _ -> align_error()
+		  _ -> align_error(Mode)
 	      end
       end, St).
 
@@ -1727,9 +1777,11 @@ align_chart(Dir, V1={X1,Y1,_},V2={X2,Y2,_}, We) ->
     Center = e3d_vec:average(V1,V2),
     rotate_chart(-Deg,Center,We).
 
--spec align_error() -> no_return().
-align_error() ->
-    wings_u:error_msg(?__(1,"Select two vertices or one edge")).
+-spec align_error(term()) -> no_return().
+align_error(Mode) when Mode==edge; Mode==vertex ->
+    wings_u:error_msg(?__(1,"Select two vertices or one edge"));
+align_error(body) ->
+    wings_u:error_msg(?__(2,"Select at least two charts to be aligned to each other")).
 
 flip_horizontal(We) ->
     flip(e3d_mat:scale(-1.0, 1.0, 1.0), We).
@@ -1761,30 +1813,49 @@ move_to(Dir,We) ->
     T = e3d_mat:translate(Translate),
     wings_we:transform_vs(T, We).
 
+align(Dir,[{Xa,Ya,_},{Xb,Yb,_}],We) ->
+    [{X1,Y1,_},{X2,Y2,_}] = wings_vertex:bounding_box(We),
+    Translate =
+        case Dir of
+            bottom ->   {0.0,(Ya-Y1),0.0};
+            top ->      {0.0,(Yb-Y2),0.0};   
+            left ->     {(Xa-X1),0.0,0.0};
+            right ->    {(Xb-X2),0.0,0.0}
+        end,
+    T = e3d_mat:translate(Translate),
+    wings_we:transform_vs(T, We).
+
 stretch(Dir,We) ->
     [{X1,Y1,_},{X2,Y2,_}] = wings_vertex:bounding_box(We),
     Center = {CX,CY,CZ} = {X1+(X2-X1)/2, Y1+(Y2-Y1)/2, 0.0},
     T0 = e3d_mat:translate(e3d_vec:neg(Center)),
     SX0 = 1.0/(X2-X1), SY0= 1.0/(Y2-Y1),
     {SX,SY} = case Dir of
-		  max_x -> {SX0, 1.0};
-		  max_y -> {1.0, SY0};
-		  max_uniform when SX0 < SY0 -> 
-		      {SX0, SX0};
-		  _ ->
-		      {SY0, SY0}
-	      end,
+                max_x -> {SX0, 1.0};
+                max_y -> {1.0, SY0};
+                max_uniform ->
+                    if SX0 < SY0 -> {SX0, SX0};
+                    true -> {SY0, SY0}
+                    end;
+                {max_uniform, Axis} ->
+                    case Axis of
+                        x -> {SX0, SX0};
+                        y -> {SY0, SY0}
+                    end
+              end,
     Stretch = e3d_mat:scale(SX, SY, 1.0),
     T1 = e3d_mat:mul(Stretch, T0),
     Pos = case Dir of
-	      max_uniform -> {0.5,0.5,CZ};
-	      max_x -> {0.5,CY,CZ};
-	      max_y -> {CX,0.5,CZ}
-	  end,    
+              {max_uniform,x} -> {CY,CY,CZ};
+              {max_uniform,y} -> {CX,CX,CZ};
+              max_uniform -> {0.5,0.5,CZ};
+              max_x -> {0.5,CY,CZ};
+              max_y -> {CX,0.5,CZ}
+          end,
     T = e3d_mat:mul(e3d_mat:translate(Pos), T1),
     wings_we:transform_vs(T, We).
     
-reunfold(Method,#st{sel=Sel,selmode=vertex}=St0) ->
+remap(Method,#st{sel=Sel,selmode=vertex}=St0) ->
     %% Check correct pinning.
     Ch = fun(Vs, #we{vp=Vtab}, _) ->
 		 case gb_sets:size(Vs) of
@@ -1819,24 +1890,28 @@ reunfold(Method,#st{sel=Sel,selmode=vertex}=St0) ->
 		{remap(Method, Pinned, Vs, We, St0),I+1}
 	end,
     {St,_} = wings_sel:mapfold(R, 1, St0),
-    wings_pb:done(update_selected_uvcoords(St)).
-
+    wings_pb:done(update_selected_uvcoords(St));
 remap(Method, #st{sel=Sel}=St0) ->
-    wings_pb:start(?__(1,"remapping")),
+    wings_pb:start(?__(3,"remapping")),
     wings_pb:update(0.001),
     N = length(Sel),
     Remap = fun(Elems, We, I) ->
-		    Msg = ?__(2,"chart")++" " ++ integer_to_list(I+1),
+		    Msg = ?__(4,"chart")++" " ++ integer_to_list(I+1),
 		    wings_pb:update(I/N, Msg),
 		    {remap(Method, none, Elems, We, St0),I+1}
 	    end,
     {St,_} = wings_sel:mapfold(Remap, 1, St0),
     wings_pb:done(update_selected_uvcoords(St)).
 
+remap(proj_lsqcm, Pinned, Sel, We, St) ->
+    remap({proj,lsqcm}, Pinned, Sel, We, St);
+remap(proj_slim, Pinned, Sel, We, St) ->
+    remap({proj,slim}, Pinned, Sel, We, St);
+
 remap(stretch_opt, _, _, We, St) ->
     Vs3d = orig_pos(We, St),
     ?SLOW(auv_mapping:stretch_opt(We, Vs3d));
-remap(proj_lsqcm, _, Sel, We0, St = #st{selmode=face}) ->
+remap({proj,Method}, _, Sel, We0, St = #st{selmode=face}) ->
     Vs3d = orig_pos(We0, St),
     try
 	Fs0   = gb_sets:to_list(Sel),
@@ -1851,7 +1926,7 @@ remap(proj_lsqcm, _, Sel, We0, St = #st{selmode=face}) ->
 			      {S,T,_} = array:get(V, Vtab),
 			      {V,{S,T}}
 			  end || V <- SelVs],
-		remap(lsqcm, Pinned, Sel, We0, St)
+		remap(Method, Pinned, Sel, We0, St)
 	end
     catch throw:{_,What} ->
 	    wpa:error_msg(What);
@@ -1920,7 +1995,7 @@ draw_background(#st{bb=#uvstate{bg_img=Image}}) ->
     %% Draw the background texture.
     gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
     gl:color3f(1.0, 1.0, 1.0),			%Clear
-    case get({?MODULE,show_background}) of
+    case ?GET({?MODULE,show_background}) of
 	false -> ok;
 	_ ->
             case wings_image:txid(Image) of
@@ -1948,6 +2023,9 @@ init_drawarea() ->
     {{W,75},{W,H0-100}}.
 
 cleanup_before_exit() ->
+    %% Ensure the used vbo data will be released. By canceling the Option
+    %% dialog doesn't remove it automatically.
+    auv_texture:delete_preview_vbo(),
     wings:unregister_postdraw_hook(wings_wm:this(), ?MODULE),
     wings_dl:delete_dlists().
 
