@@ -15,7 +15,7 @@
 
 -export([init/0,menu/2,command/2]).
 
--include_lib("wings.hrl").
+-include_lib("wings/src/wings.hrl").
 
 init() ->
     true.
@@ -81,7 +81,7 @@ doo_sabin(#we{es=Etab0}=We0) ->
     %% Return updated #we{} and selection.
     {We5,CornerEdges}.
 
-%% Disolve edges and isolated vertices.
+%% Dissolve edges and isolated vertices.
 dissolve_edges(Es, We0) ->
     We1 = wings_edge:dissolve_edges(Es, We0),
     IsolatedVs = wings_vertex:isolated(We1),
