@@ -13,7 +13,7 @@
 
 -module(wpc_move_planar).
 -export([init/0,menu/2,command/2]).
--include("wings.hrl").
+-include_lib("wings/src/wings.hrl").
 
 init() ->
     true.
