@@ -236,13 +236,8 @@ handle_magnet_event(#mousemotion{x=X}, X0, Y0, Sc0) ->
     wings_io:warp(X0,Y0),
     Sc = adjust_magnet_radius(DX, Sc0),
     update_magnet_handler(X0, Y0, Sc);
-handle_magnet_event(#mousebutton{button=4,state=?SDL_RELEASED}, X, Y, Sc0) ->
-    update_magnet_handler(X, Y, adjust_strength(1, Sc0));
-handle_magnet_event(#mousebutton{button=5,state=?SDL_RELEASED}, X, Y, Sc0) ->
-    update_magnet_handler(X, Y, adjust_strength(-1, Sc0));
-handle_magnet_event(#mousebutton{button=Button}, X, Y, Sc)
-  when Button =:= 4; Button =:= 5 ->
-    update_magnet_handler(X, Y, Sc);
+handle_magnet_event(#mousewheel{dir=ver,wheel=N}, X, Y, Sc0) ->
+    update_magnet_handler(X, Y, adjust_strength(N, Sc0));
 handle_magnet_event(#mousebutton{}=Ev, X, Y, Sc) ->
     wings_wm:later(Ev),
     end_magnet_event(X, Y, Sc);
@@ -330,9 +325,8 @@ draw_magnet(X, Y, #sculpt{rad=Rad,str=Str,st=#st{shapes=Shs}=St}) ->
     P = Str*10,
     wings_io:set_color({1.0*P,0.0,1.0/P,0.1}),
     gl:translatef(Xm, Ym, Zm),
-    Obj = glu:newQuadric(),
-    glu:sphere(Obj, Rad, 40, 40),
-    glu:deleteQuadric(Obj),
+    #{size:=Size, tris:=Tris} = wings_shapes:tri_sphere(#{subd=>4, scale=> Rad, binary => true}),
+    wings_vbo:draw(fun(_) -> gl:drawArrays(?GL_TRIANGLES, 0, Size*3) end, Tris),
     gl:popAttrib().
 
 dist_along_vector({Xa,Ya,Za},{Xb,Yb,Zb},{Vx,Vy,Vz}) ->
@@ -1166,8 +1160,8 @@ draw(plain, DrawEdges, _D, SelMode, RS) ->
     wings_dl:call(DrawEdges, RS);
 draw(_,_,_,_, RS) -> RS.
 
-edge_width(edge) -> wings_pref:get_value(edge_width);
-edge_width(_) -> 1.
+edge_width(edge) -> float(wings_pref:get_value(edge_width));
+edge_width(_) -> 1.0.
 
 col_to_vec({R,G,B}) when is_integer(R) -> {R/255.0,G/255.0,B/255.0};
 col_to_vec({_,_,_}=Col) -> Col;
