@@ -105,8 +105,9 @@ export_1(Filename, Contents0, Attr) ->
                end,
     ok = filelib:ensure_dir(filename:join(ImageDir, "dummy")),
     Imagetype = proplists:get_value(default_filetype, Attr, ?DEF_IMAGE_TYPE),
-    Contents1 = wpa:save_images(Contents0, ImageDir, Imagetype),
-    #e3d_file{objs=Objs, mat=Mat, creator=Creator} = Contents1,
+    Contents1 = e3d_file:transform(Contents0, wpa:export_matrix(Attr)),
+    Contents2 = wpa:save_images(Contents1, ImageDir, Imagetype),
+    #e3d_file{objs=Objs, mat=Mat, creator=Creator} = Contents2,
     GLTF0 = #{asset => #{generator => unicode:characters_to_binary(Creator),
                          version=> <<"2.0">>},
               accessors   => [],
@@ -206,9 +207,9 @@ exp_add_index(Ps0, GLTF0) ->
     {Ps1, Bin} = lists:mapfoldl(AppendBin, <<>>, Ps0),
     {BVId, GLTF1} = exp_add(Bin, bufferViews, GLTF0),
     lists:mapfoldr(fun({#{indices:=Len}=P, Offset}, GLTF_t) ->
-                           Acces = exp_make_acc(BVId, Len, ?GL_UNSIGNED_INT,
+                           Access = exp_make_acc(BVId, Len, ?GL_UNSIGNED_INT,
                                                 <<"SCALAR">>, Offset),
-                           {AId, GLTF} = exp_add(Acces, accessors, GLTF_t),
+                           {AId, GLTF} = exp_add(Access, accessors, GLTF_t),
                            {P#{indices:=AId}, GLTF}
                    end, GLTF1, Ps1).
 
@@ -954,8 +955,8 @@ split_tx(OrigFileName, Dir0) ->
         RoughFile = filename:join(Dir, File++"_rough" ++ Ext),
         wings_image:image_write([{filename, RoughFile}, {image, Roughness}]),
         {MetFile, RoughFile}
-    catch _:Reason ->
-            ?dbg("Internal error: ~p ~p~n", [Reason, erlang:get_stacktrace()]),
+    catch _:Reason:ST ->
+            ?dbg("Internal error: ~p ~p~n", [Reason, ST]),
             {none, none}
     end.
 
