@@ -200,7 +200,7 @@ new_viewer(Name, Pos, Size, Props, St) ->
     Op = main_loop_noredraw(St),
     Title = geom_title(Name),
     {Frame,Ps} = wings_frame:make_win(Title, [{size, Size}, {pos, Pos}|Props]),
-    Context = wxGLCanvas:getContext(?GET(gl_canvas)),
+    Context = ?GET(gl_context),
     Show = proplists:is_defined(external, Ps),
     Canvas = wings_gl:window(Frame, Context, true, Show),
     wings_wm:toplevel(Name, Canvas, Ps ++ initial_properties(), Op),
@@ -422,6 +422,7 @@ handle_event_3({vec_command,Command,St}, _) when is_function(Command) ->
     command_response(Command(), none, St);
 handle_event_3(#mousebutton{}, _St) -> keep;
 handle_event_3(#mousemotion{}, _St) -> keep;
+%handle_event_3(#mousewheel{}, _St) -> keep;
 handle_event_3(init_opengl, St) ->
     init_opengl(St),
     main_loop_noredraw(St);
@@ -499,6 +500,8 @@ handle_event_3(#wx{event=#wxDropFiles{files=Fs0}}, St0) ->
     lists:foldl(Open, DoOpen, Objects),
     Send({import_files, Fs2}),
     keep;
+handle_event_3({drop,_}, _St) ->
+    keep;
 handle_event_3(ignore, _St) ->
     keep;
 handle_event_3({system,_,_}, _St) ->
@@ -510,12 +513,16 @@ handle_event_3({command_error, Error}, _St) ->
 handle_event_3({adv_menu_abort, Ev}, _St) ->
     This = wings_wm:actual_focus_window(),
     wings_wm:send(This,Ev);
-handle_event_3({camera,Ev,NextEv}, St) ->
+handle_event_3({camera,Ev0,NextEv}, St) ->
     %% used by preview dialogs in (blanket event)
     {_,X,Y} = wings_wm:local_mouse_state(),
-    case wings_camera:event(Ev#mousebutton{x=X,y=Y}, St) of
-      next -> NextEv;
-      Other -> Other
+    Ev = case Ev0 of
+             #mousewheel{} -> Ev0#mousewheel{x=X,y=Y};
+             #mousebutton{} -> Ev0#mousebutton{x=X,y=Y}
+         end,
+    case wings_camera:event(Ev, St) of
+        next -> NextEv;
+        Other -> Other
     end.
 
 handle_popup_event(Ev, Xglobal, Yglobal, St0) ->
@@ -661,8 +668,8 @@ execute_command(Cmd, Ev, St) ->
 	wings_menu:check_item(Cmd),
 	command(Cmd, St)
     catch
-	error:Reason ->
-	    io:format("~p:~p: Error: ~p ~p~n", [?MODULE, ?LINE, Reason, erlang:get_stacktrace()]),
+	error:Reason:ST ->
+	    io:format("~p:~p: Error: ~p ~p~n", [?MODULE, ?LINE, Reason, ST]),
 	    wings_hotkey:handle_error(Ev, Cmd),
 	    St#st{repeatable=ignore}
     end.
@@ -836,6 +843,8 @@ command_1({window,outliner}, St) ->
     wings_outliner:window(St);
 command_1({window,object}, St) ->
     wings_geom_win:window(St);
+command_1({window,view}, St) ->
+    wings_view_win:window(St);
 command_1({window,palette}, St) ->
     wings_palette:window(St);
 command_1({window,console}, _St) ->
@@ -1032,6 +1041,7 @@ window_menu() ->
 	     {Name,object,
 	      ?__(5,"Open a Geometry Graph window (showing objects)")},
 	     {?__(6,"Palette"), palette,?__(7,"Open the color palette window")},
+             {?__(14,"View Settings"), view, ?__(15,"Open view settings window")},
 	     {?__(12,"Tweak Palette"), tweak_palette,
 	      ?__(13,"Open palettes from which tweak tools may be selected or bound to modifier keys")},
 	     separator,
