@@ -212,6 +212,7 @@ save_window(palette) -> true;
 save_window({tweak, _}) -> true;
 save_window(outliner) -> true;
 save_window(wings_outliner) -> true;
+save_window(wings_view_win) -> true;
 save_window({object,_}) -> true;
 save_window(geom) -> true;
 save_window({geom,_}) -> true;
@@ -267,6 +268,8 @@ restore_window({{object,_}=Name,Pos,{_,_}=Size,Ps}, St) ->
     wings_geom_win:window(Name, Pos, Size, Ps, St);
 restore_window({wings_outliner,Pos,{_,_}=Size, Ps}, St) ->
     wings_outliner:window(Pos, Size, Ps, St);
+restore_window({wings_view_win,Pos,{_,_}=Size, Ps}, St) ->
+    wings_view_win:window(Pos, Size, Ps, St);
 restore_window({palette,Pos,{_,_}=Size, Ps}, St) ->
     wings_palette:window(Pos, Size, Ps, St);
 restore_window({console,Pos,{_,_}=Size, Ps}, _St) ->
@@ -346,8 +349,8 @@ init(_Opts) ->
         wings ! {frame_created, Frame},
         receive opengl_initialized -> ok end,
 	{Frame, #state{toolbar=Toolbar, images=IconImgs, windows=Wins, overlay=Overlay}}
-    catch _:Reason ->
-	    io:format("CRASH: ~p ~p ~p~n",[?MODULE, Reason, erlang:get_stacktrace()])
+    catch _:Reason:ST ->
+	    io:format("CRASH: ~p ~p ~p~n",[?MODULE, Reason, ST])
     end.
 
 make_splash(Canvas, Imgs) ->
@@ -366,11 +369,18 @@ make_splash(Canvas, Imgs) ->
 
 %%%%%%%%%%%%%%%%%%%%%%%
 
-handle_event(#wx{obj=Win, event=#wxMove{}}, State) ->
+handle_event(#wx{obj=Obj, event=#wxMove{}}, #state{windows=#{loose:=Loose}} = State) ->
     MS = wx_misc:getMouseState(),
     {X,Y} = wx_misc:getMousePosition(),
-    %% io:format("Move ~p ~p ~p ~n", [Win, X,Y]),
-    {noreply, preview_attach(stopped_moving(MS), {X,Y}, Win, State)};
+    Find = fun(#win{frame=Frame, name=Name}) ->
+                   wings_util:wxequal(Frame, Obj) andalso save_window(Name)
+           end,
+    case lists:any(Find, maps:values(Loose)) of
+        true ->
+            {noreply, preview_attach(stopped_moving(MS), {X,Y}, Obj, State)};
+        false ->
+            {noreply, State}
+    end;
 
 handle_event(#wx{userData={move,Win}, event=Ev}, #state{windows=Wins0} = State) ->
     Wins = detach_window(Ev, Win, Wins0),
@@ -520,6 +530,7 @@ handle_cast({mode_restriction, _}=Restrict, #state{toolbar=TB}=State) ->
 handle_cast({menu, {Menu, Key, Value}=Update}, #state{toolbar=TB}=State) ->
     wings_menu:update_menu_enabled(Menu, Key, Value),
     wings_toolbar:update(Update, TB),
+    wings_view_win:update(Update),
     {noreply, State};
 handle_cast({got_focus, Window, Props}, #state{toolbar=TB0}=State) ->
     Fun = fun(Menu, Key, Value) ->
@@ -584,6 +595,7 @@ code_change(_From, _To, State) ->
 terminate(_Reason, #state{windows=#{frame:=Frame}}) ->
     %% io:format("~p: terminate: ~p~n",[?MODULE, _Reason]),
     catch wxFrame:destroy(Frame),
+    wx:destroy(),
     shutdown.
 
 terminate_frame(_Ev, CB) ->
@@ -678,9 +690,11 @@ preview_attach(false, Pos, Frame,
   when Action =:= undefined; Action =:= preview_attach ->
     case get_split_path(Pos, Wins) of
 	ignore -> %% Outside attached window
+            wings_status:message(?MODULE, ""),
 	    overlay_hide(Overlay),
 	    State#state{windows=delete_timer(Wins)};
 	Path ->
+            wings_status:message(?MODULE, ?__(1, "Hold SHIFT to not attach window")),
 	    {Rect,_} = preview_rect(Path, Frame),
 	    overlay_draw(Overlay, Rect, 170),
 	    State#state{windows=setup_timer(Frame, Path, delete_timer(Wins))}
@@ -711,12 +725,12 @@ preview_rect({Obj, Path}, Frame) ->
 
 make_overlay(Parent) ->
     Flags = ?wxFRAME_TOOL_WINDOW bor
-	?wxFRAME_FLOAT_ON_PARENT bor
+	?wxSTAY_ON_TOP bor
 	?wxFRAME_NO_TASKBAR bor
 	?wxNO_BORDER,
     Overlay = wxFrame:new(),
     case {os:type(), {?wxMAJOR_VERSION, ?wxMINOR_VERSION}} of
-        {{_, linux}, Ver} when Ver > {3,0} ->
+        {{_, linux}, Ver} when Ver >= {3,0} ->
             wxFrame:setBackgroundStyle(Overlay, 3); %% ?wxBG_STYLE_TRANSPARENT
         _ -> ok
     end,
@@ -735,6 +749,7 @@ overlay_hide(Overlay) ->
 attach_floating(false, _, #{op:=Op}=State) when Op =/= undefined ->
     setup_timer(State);
 attach_floating(true, Overlay, #{op:=#{mwin:=Frame, mpath:=Path}, loose:=Loose}=State) ->
+    wings_status:message(?MODULE, ""),
     case maps:get(Frame, Loose, false) of
 	false ->
 	    State#{action:=undefined, op:=undefined};
@@ -752,6 +767,7 @@ attach_floating(true, Overlay, #{op:=#{mwin:=Frame, mpath:=Path}, loose:=Loose}=
     end;
 attach_floating(_B, Overlay, State) ->
     %% Spurious Move events on windows
+    wings_status:message(?MODULE, ""),
     overlay_hide(Overlay),
     State#{action:=undefined, op:=undefined}.
 
@@ -790,7 +806,11 @@ split_win([Which|Path], NewWin, #split{mode=Mode} = Node, Pos) ->
     {Child0, Child1} = child(Which, Node),
     Child = split_win(Path, NewWin, Child0, Pos),
     wxSplitterWindow:replaceWindow(win(Node), win(Child0), win(Child)),
-    set_splitter(Which, Mode, false, Node, Child1, Child).
+    set_splitter(Which, Mode, false, Node, Child1, Child);
+split_win([_|Path], NewWin, Node, Pos0) ->
+    %% Something went wrong here a plugin or window we don't know about
+    %% in this release was not created as it should so the path was incorrect
+    split_win(Path, NewWin, Node, Pos0).
 
 make(Parent) ->
     Style = case os:type() of
@@ -1306,8 +1326,8 @@ tree_to_list(#win{name=Name}, _, Acc) ->
 check_tree(#split{} = T, Orig) ->
     try
 	true = tree_consistent(T)
-    catch _:Reason ->
-	    io:format("Failed ~p ~p~n", [Reason, erlang:get_stacktrace()]),
+    catch _:Reason:ST ->
+	    io:format("Failed ~p ~p~n", [Reason, ST]),
 	    io:format("Orig ~p~nNew ~p~n",[Orig, T])
     end;
 check_tree(T, Orig) ->
@@ -1345,18 +1365,21 @@ make_icons() ->
 init_menubar(Frame) ->
     ets:new(wings_menus, [named_table, public, {keypos,2}]),
     put(wm_active, {menubar, geom}),
-    WorkAround = try
-                     %% Only exists in future wx (erlang release)
-                     %% and fool dialyzer
-                     WxMB = list_to_atom("wxMenuBar"),
-                     WxMB:setAutoWindowMenu(false),
-                     false
-                 catch _:_ ->
-                         case os:type() of
-                             {_, darwin} -> true;
-                             _ -> false
-                         end
-                 end,
+    WorkAround =
+        case os:type() of
+            {_, darwin} ->
+                try
+                    %% Fool dialyzer
+                    WxMB = list_to_atom("wxMenuBar"),
+                    WxMB:setAutoWindowMenu(false),
+                    false
+                catch _:_ ->
+                        %% Exists in newer wx (erlang release)
+                        true
+                end;
+            _ ->
+                false
+        end,
     MB = wxMenuBar:new(),
     wings_menu:setup_menus(MB, top_menus(WorkAround)),
     wxFrame:setMenuBar(Frame, MB),
