@@ -13,8 +13,8 @@
 
 -module(wings_gl).
 -export([init/1, window/4, attributes/0,
+         setCurrent/2, wxGLCanvas_new/3, %% new wx api variants
 	 is_ext/1,is_ext/2,
-	 is_restriction/1,
 	 error_string/1]).
 
 %% GLSL exports
@@ -27,7 +27,7 @@
 -export([have_fbo/0, setup_fbo/2, delete_fbo/1]).
 
 %% GL wrappers
--export([project/6, unProject/6,
+-export([
 	 triangulate/2, deleteTextures/1,
 	 bindFramebuffer/2,
 	 drawElements/4
@@ -47,7 +47,6 @@
 init(Parent) ->
     GL = window(Parent, undefined, true, false),
     init_extensions(),
-    init_restrictions(),
     GL.
 
 attributes() ->
@@ -70,40 +69,50 @@ attributes() ->
          SB ++ [0]
     }.
 
-window(Parent, Context, Connect, Show) ->
+window(Parent, Context0, Connect, Show) ->
     Style = ?wxFULL_REPAINT_ON_RESIZE bor ?wxWANTS_CHARS bor wings_frame:get_border(),
     Flags = [attributes(), {style, Style}],
-    GL = case Context of
-	     undefined ->
-		 Win = wxGLCanvas:new(Parent, Flags),
-		 ?SET(gl_canvas, Win),
-		 Win;
-	     _ ->
-		 wxGLCanvas:new(Parent, Context, Flags)
-	 end,
-    Connect andalso connect_events(GL),
+    GL = wxGLCanvas_new(Parent, Context0, Flags),
+    case Context0 of
+        undefined ->
+            Context = wxGLContext_new(GL),
+            %% If top gl window, store context and canvas
+            ?SET(gl_canvas, GL),
+            ?SET(gl_context, Context);
+        Context0 ->
+            Context = Context0
+    end,
+
+    wxGLCanvas:setBackgroundStyle(GL, ?wxBG_STYLE_PAINT),
+    Connect andalso connect_events(GL, Context),
     case Show of
 	true ->
 	    wxWindow:connect(Parent, show),
 	    wxFrame:show(Parent),
 	    receive #wx{event=#wxShow{}} -> ok end;
 	false ->
-            timer:sleep(200), %% Let wx realize the window on gtk
 	    ok
     end,
+    %% Let wxWidgets have time to realize the window (on GTK)
+    %% otherwise the setCurrent fails.
+    %% The show event may come before the window is actually
+    %% displayed, sigh, so always sleep for a short while before
+    %% the setCurrent call.
+    timer:sleep(200),
     wxWindow:disconnect(Parent, show),
-    wxGLCanvas:setCurrent(GL),
+    setCurrent(GL,Context),
     GL.
 
 %% Event handling for OpenGL windows
 
-connect_events(Canvas) ->
+connect_events(Canvas, Context) ->
     %% Re-attaches the OpenGL Context to Window when is [re] created/showed
     wxWindow:connect(Canvas, create,
-                     [{callback, fun(_, _) -> wxGLCanvas:setCurrent(Canvas) end}]),
+                     [{callback, fun(_, _) -> setCurrent(Canvas, Context) end}]),
     wxWindow:connect(Canvas, show,
                      [{callback, fun(#wx{event=#wxShow{show=Show}}, _) ->
-                                         Show andalso (catch wxGLCanvas:setCurrent(Canvas)) end}]),
+                                         Show andalso (catch setCurrent(Canvas, Context))
+                                 end}]),
     case os:type() of
 	{unix, darwin} ->
 	    wxWindow:connect(Canvas, paint, [{callback, fun redraw/2}]);
@@ -111,8 +120,7 @@ connect_events(Canvas) ->
 	    wxWindow:connect(Canvas, paint, [{skip, true}]),
 	    ok;
 	{win32, _} ->
-	    wxWindow:connect(Canvas, paint, [{callback, fun redraw/2}]),
-	    wxWindow:connect(Canvas, erase_background, [{callback, fun redraw/2}])
+	    wxWindow:connect(Canvas, paint, [{callback, fun redraw/2}])
     end,
 
     wxWindow:connect(Canvas, size),
@@ -129,9 +137,7 @@ redraw(#wx{obj=Canvas, event=#wxPaint{}}=Ev,_) ->
     DC = wxPaintDC:new(Canvas),
     wxPaintDC:destroy(DC),
     wings ! Ev,
-    ok;
-redraw(Ev, _) ->  %% For erase background events
-    wings ! Ev#wx{event=#wxPaint{type=paint}}.
+    ok.
 
 setup_std_events(Canvas) ->
     wxWindow:connect(Canvas, motion),
@@ -142,7 +148,17 @@ setup_std_events(Canvas) ->
     wxWindow:connect(Canvas, left_dclick),
     wxWindow:connect(Canvas, right_up),
     wxWindow:connect(Canvas, right_down),
-    wxWindow:connect(Canvas, mousewheel),
+    case code:is_loaded(wxMouseEvent) of
+        false -> code:load_file(wxMouseEvent);
+        _ -> ok
+    end,
+    case erlang:function_exported(wxMouseEvent, getWheelAxis, 1) of
+        true ->
+            wxWindow:connect(Canvas, mousewheel,
+                             [{callback, fun wings_io_wx:scroll_event/2}]);
+        false ->
+            wxWindow:connect(Canvas, mousewheel)
+    end,
     %% wxWindow:connect(Canvas, char_hook, []),
     wxWindow:connect(Canvas, key_down, [{callback, fun key_callback_win32/2}]),
     wxWindow:connect(Canvas, key_up), %% Normally suppressed
@@ -169,6 +185,37 @@ forward_key(#wxKey{metaDown=true}) -> true;
 forward_key(#wxKey{shiftDown=true, keyCode=?WXK_SHIFT}) -> true;
 forward_key(_) -> false.
 
+new_gl_api() ->
+    wings_u:is_exported(wxGLCanvas, setCurrent, 2).
+
+setCurrent(GL,Context) ->
+    SetCurrent = wings_u:id(setCurrent),
+    case new_gl_api() of
+        false -> wxGLCanvas:SetCurrent(GL);
+        true -> wxGLCanvas:SetCurrent(GL,Context)
+    end.
+
+
+wxGLCanvas_new(Parent, undefined, Ps) ->
+    wxGLCanvas:new(Parent, Ps);
+wxGLCanvas_new(Parent, Context, Ps) ->
+    New = wings_u:id(new),
+    case new_gl_api() of
+        false ->
+            wxGLCanvas:New(Parent, Context, Ps);
+        true ->
+            wxGLCanvas:New(Parent, Ps)
+    end.
+
+wxGLContext_new(Canvas) ->
+    Context = wings_u:id(wxGLContext),
+    GetContext = wings_u:id(getContext),
+    case new_gl_api() of
+        false ->
+            wxGLCanvas:GetContext(Canvas);
+        true ->
+            Context:new(Canvas)
+    end.
 
 %%%
 %%% OpenGL extensions.
@@ -229,22 +276,6 @@ version_match({Ma1,Mi1,P1}, {Ma2,Mi2,P2})
 version_match(_,_) ->
     false.
 
-%%%
-%%% OpenGL restrictions (bugs and limitations).
-%%%
-init_restrictions() ->
-    ets:new(wings_gl_restriction, [named_table,public,ordered_set]),
-    case os:type() of
-	{unix,sunos} ->
-	    %% Scissor does not work for clipping text.
-	    ets:insert(wings_gl_restriction, [{broken_scissor}]);
-	_ ->
-	    ok
-    end.
-
-is_restriction(Name) ->
-    ets:member(wings_gl_restriction, Name).
-
 error_string(0) -> no_error;
 error_string(?GL_INVALID_VALUE) -> "GL_INVALID_VALUE";
 error_string(?GL_INVALID_ENUM) -> "GL_INVALID_ENUM";
@@ -406,14 +437,14 @@ setup_fbo_2({color, Options}, {W,H}, Count) ->
     gl:texImage2D(?GL_TEXTURE_2D, 0, Internal, W, H, 0, Format, Type, 0),
 
     MinF = proplists:get_value(min, Options, ?GL_LINEAR),
-    gl:texParameterf(?GL_TEXTURE_2D,?GL_TEXTURE_MIN_FILTER, MinF),
+    gl:texParameteri(?GL_TEXTURE_2D,?GL_TEXTURE_MIN_FILTER, MinF),
     MagF = proplists:get_value(mag, Options, ?GL_LINEAR),
-    gl:texParameterf(?GL_TEXTURE_2D, ?GL_TEXTURE_MAG_FILTER, MagF),
+    gl:texParameteri(?GL_TEXTURE_2D, ?GL_TEXTURE_MAG_FILTER, MagF),
 
     WS = proplists:get_value(wrap_s, Options, ?GL_REPEAT),
-    gl:texParameterf(?GL_TEXTURE_2D, ?GL_TEXTURE_WRAP_S, WS),
+    gl:texParameteri(?GL_TEXTURE_2D, ?GL_TEXTURE_WRAP_S, WS),
     WT = proplists:get_value(wrap_t, Options, ?GL_REPEAT),
-    gl:texParameterf(?GL_TEXTURE_2D, ?GL_TEXTURE_WRAP_T, WT),
+    gl:texParameteri(?GL_TEXTURE_2D, ?GL_TEXTURE_WRAP_T, WT),
 
     GEN_MM = proplists:get_value(gen_mipmap, Options, ?GL_FALSE),
     gl:texParameteri(?GL_TEXTURE_2D, ?GL_GENERATE_MIPMAP, GEN_MM),
@@ -484,28 +515,9 @@ check_fbo_status(FB) ->
 bindFramebuffer(W, Fbo) ->
     gl:bindFramebuffer(W,Fbo).
 
-project(X,Y,Z, Mod, Proj, View) ->
-    {_, RX,RY,RZ} = glu:project(X,Y,Z, mat(Mod), mat(Proj), View),
-    {RX,RY,RZ}.
-
-unProject(X,Y,Z, Mod, Proj, View) ->
-    {_, RX,RY,RZ} = glu:unProject(X,Y,Z, mat(Mod), mat(Proj), View),
-    {RX,RY,RZ}.
-
-mat(Mat) when tuple_size(Mat) =:= 16 ->  Mat;
-mat(Mat) when is_tuple(Mat) -> e3d_mat:expand(Mat);
-mat(List) when is_list(List) ->  mat(list_to_tuple(List)).
-
 triangulate(Normal, Pos0) ->
-    {Tris0, BinPos} = glu:tesselate(Normal, Pos0),
-    Tris = tris(Tris0),
-    Res = {Tris, [{X,Y,Z}|| <<X:64/float-native,Y:64/float-native, Z:64/float-native>> <= BinPos]},
-    %%io:format("~p~n~p~n~p~n",[Pos0, Tris, element(2, Res)]),
-    Res.
+    wings_glu_tess:triangulate(Normal, Pos0).
 
-tris([A,B,C|Rs]) ->
-    [{A+1,B+1,C+1}|tris(Rs)];
-tris([]) -> [].
 
 deleteTextures(List) ->
     gl:deleteTextures(List).
