@@ -100,23 +100,22 @@ tex_id(#font{tex=TexId}) ->
 render(#font{} = GLFont, String) when is_list(String) ->
     render_text(GLFont, String);
 render(#font{}, {_, _, <<>>}) -> ok;
-render(#font{tex=TexId, height=H}, {X,Y, Bin0}) ->
-    Size = byte_size(Bin0),
-    Bin = <<_:2/unit:32, TxBin/bytes>> =
-	if Size < ?BIN_XTRA -> <<Bin0/bytes, 0:(?BIN_XTRA*8)>>;
-	   true -> Bin0
-	end,
-
-    wx:retain_memory(Bin),
+render(#font{tex=TexId, height=H}, {X,Y, _, Data}) ->
+    Size = byte_size(Data),
+    [Vbo] = Buffers = gl:genBuffers(1),
+    gl:bindBuffer(?GL_ARRAY_BUFFER, Vbo),
+    gl:bufferData(?GL_ARRAY_BUFFER, Size, Data, ?GL_STATIC_DRAW),
     gl:bindTexture(?GL_TEXTURE_2D, TexId),
-    gl:vertexPointer(2, ?GL_FLOAT, 16, Bin),
-    gl:texCoordPointer(2, ?GL_FLOAT, 16, TxBin),
+
+    gl:vertexPointer(2, ?GL_FLOAT, 16, 0),
+    gl:texCoordPointer(2, ?GL_FLOAT, 16, 8),
     gl:enableClientState(?GL_VERTEX_ARRAY),
     gl:enableClientState(?GL_TEXTURE_COORD_ARRAY),
     gl:drawArrays(?GL_QUADS, 0, Size div 16),
     gl:disableClientState(?GL_VERTEX_ARRAY),
     gl:disableClientState(?GL_TEXTURE_COORD_ARRAY),
-    wx:release_memory(Bin),
+    gl:bindBuffer(?GL_ARRAY_BUFFER, 0),
+    gl:deleteBuffers(Buffers),
     {X,abs(Y-H)}.
 
 %% @spec(Font::font_info(), unicode:charlist()) ->
@@ -141,8 +140,8 @@ render_to_binary(#font{glyphs=Gs, height=H, ih=IH, iw=IW}, String) ->
 %%     gl:enableClientState(?GL_TEXTURE_COORD_ARRAY),</br>
 %%     gl:drawArrays(?GL_QUADS, 0, (byte_size(Bin)-?BIN_XTRA) div 16),</br>
 render_to_binary(#font{glyphs=Gs, height=H, ih=IH, iw=IW},
-		 String, Data = {X,Y,D})
-  when is_integer(X), is_integer(Y), is_binary(D) ->
+		 String, Data = {X,Y,XS,D})
+  when is_integer(X), is_integer(Y), is_integer(XS), is_binary(D) ->
     render_text3(String, Gs, IH, IW, H, Data).
 
 %%--------------------------------------------------------------------
@@ -267,17 +266,17 @@ gen_texture(TW,TH,Bin,Options) ->
 
     %% gl:pixelStorei(?GL_UNPACK_ALIGNMENT, 1),
     Mode = proplists:get_value(tex_mode, Options, ?GL_MODULATE),
-    gl:texEnvf(?GL_TEXTURE_ENV, ?GL_TEXTURE_ENV_MODE, Mode),
+    gl:texEnvi(?GL_TEXTURE_ENV, ?GL_TEXTURE_ENV_MODE, Mode),
 
     MinF = proplists:get_value(tex_min, Options, ?GL_LINEAR),
-    gl:texParameterf(?GL_TEXTURE_2D,?GL_TEXTURE_MIN_FILTER,MinF),
+    gl:texParameteri(?GL_TEXTURE_2D,?GL_TEXTURE_MIN_FILTER, MinF),
     MagF = proplists:get_value(tex_mag, Options, ?GL_LINEAR),
-    gl:texParameterf(?GL_TEXTURE_2D,?GL_TEXTURE_MAG_FILTER,MagF),
+    gl:texParameteri(?GL_TEXTURE_2D,?GL_TEXTURE_MAG_FILTER,MagF),
 
     WS = proplists:get_value(tex_wrap_s, Options, ?GL_CLAMP),
-    gl:texParameterf(?GL_TEXTURE_2D,?GL_TEXTURE_WRAP_S, WS),
+    gl:texParameteri(?GL_TEXTURE_2D,?GL_TEXTURE_WRAP_S, WS),
     WT = proplists:get_value(tex_wrap_t, Options, ?GL_CLAMP),
-    gl:texParameterf(?GL_TEXTURE_2D,?GL_TEXTURE_WRAP_T, WT),
+    gl:texParameteri(?GL_TEXTURE_2D,?GL_TEXTURE_WRAP_T, WT),
 
     GEN_MM = proplists:get_value(tex_gen_mipmap, Options, ?GL_FALSE),
     gl:texParameteri(?GL_TEXTURE_2D, ?GL_GENERATE_MIPMAP, GEN_MM),
@@ -300,13 +299,12 @@ memory_dc(Font) ->
     MDC.
 
 render_text(Font=#font{glyphs=Gs, height=H, ih=IH, iw=IW}, String) ->
-    Res = render_text3(String, Gs, IH, IW, H, {0,0, <<>>}),
+    Res = render_text3(String, Gs, IH, IW, H, {0,0,0, <<>>}),
     render(Font, Res).
 
-render_text3([$\n|String], Gs, IH, IW, H, Data0) ->
+render_text3([$\n|String], Gs, IH, IW, H, {_,H0,XS,Bin}) ->
     %% New line
-    {_,H0,Bin} = Data0,
-    render_text3(String, Gs, IH, IW, H, {0, H0+H, Bin});
+    render_text3(String, Gs, IH, IW, H, {XS,H0+H,XS,Bin});
 render_text3([$\t|String], Gs, IH, IW, H, Data0) ->
     %% Tab
     [{_,Space}] = ets:lookup(Gs, 32),
@@ -327,15 +325,15 @@ render_text3([Other|String], Gs, IH, IW, H, Data0) ->
     render_text3(String, Gs, IH, IW, H, Data);
 render_text3(Bin, Gs, IH, IW, H, Data) when is_binary(Bin) ->
     render_text3(unicode:characters_to_list(Bin), Gs, IH, IW, H, Data);
-render_text3([], _Gs, _IH, _IW, _H, {W,H0,Bin}) ->
-    {W, H0, Bin}.
+render_text3([], _Gs, _IH, _IW, _H, Res) ->
+    Res.
 
-render_glyph(#glyph{u=U,v=V,w=W,h=H},IW,IH, {X0,Y0,Bin}) ->
+render_glyph(#glyph{u=U,v=V,w=W,h=H},IW,IH, {X0,Y0,XS,Bin}) ->
     X1 = X0 + W,
     UD = U + W*IW,
     VD = V + H*IH,
     YH = Y0 - H,
-    {X1,Y0,
+    {X1,Y0,XS,
      <<Bin/binary,         %% wxImage: 0,0 is upper left turn each
        X0:?F32,Y0:?F32, U:?F32, VD:?F32, % Vertex lower left, UV-coord up-left
        X1:?F32,Y0:?F32, UD:?F32,VD:?F32, % Vertex lower right,UV-coord up-right
