@@ -161,63 +161,74 @@ have_magnet(_, true) -> activated;
 have_magnet(Ps, _) ->
     proplists:is_defined(magnet, Ps).
 
-mac_fix() ->
-    %% Mac wxWidgets-3.1.3 capture_mouse don't work as expected
-    %% Try to work around it.
-    case os:type() of
-        {unix, darwin} -> true;
-        _ -> false
-    end.
-
 wx_popup_menu_init(Parent,GlobalPos,Names,Menus0) ->
-    mac_fix() andalso wings_wm:grab_focus(),
-    Owner = wings_wm:this(),
-    {Pid, Entries} = wx_popup_menu(Parent,GlobalPos,Names,Menus0,false,Owner),
-    {push, fun(Ev) -> popup_event_handler(Ev, {Parent,Owner,Pid}, Entries) end}.
+    case wings_wm:grabbed_focus_window() of
+        dialog_blanket ->
+            wings_wm:send(dialog_blanket, user_attention);
+        _ ->
+            Owner = wings_wm:this(),
+            Entries = wx_popup_menu(Parent,GlobalPos,Names,Menus0,false,dialog_blanket),
+            {TopW,TopH} = wings_wm:top_size(),
+            Op = {push, fun(Ev) -> popup_event_handler(Ev, {Parent,Owner}, Entries) end},
+            wings_wm:new(dialog_blanket, {0,0,highest}, {TopW,TopH}, Op),
+            wings_wm:grab_focus(dialog_blanket)
+	end,
+    keep.
 
 wx_popup_menu(Parent,Pos,Names,Menus0,Magnet,Owner) ->
-    HotKeys = wings_hotkey:matching(Names),
-    is_list(Menus0) orelse erlang:error(Menus0),
-    Menus1   = wings_plugin:menu(list_to_tuple(reverse(Names)), Menus0),
-    Entries0 = [normalize_menu_wx(Entry, HotKeys, Names) || Entry <- lists:flatten(Menus1)],
-    Entries1 = format_hotkeys(Entries0, pretty),
-    {Entries2,_}  = lists:foldl(fun(ME, {List, Id}) ->
+    Entries0 = make_entries(Names, Menus0, pretty),
+    {Entries1,_}  = lists:foldl(fun(ME, {List, Id}) ->
 					{[ME#menu{wxid=Id},
 					  ME#menu{wxid=Id+1, type=opt}|List],
 					 Id+2}
-				end, {[],500}, Entries1),
-    Entries = reverse(Entries2),
-    MEs0 = [ME#menu{name=undefined} || ME <- Entries],
-    CreateMenu = fun() -> setup_dialog(Parent, MEs0, Magnet, Pos) end,
+				end, {[],500}, Entries0),
+    Entries = reverse(Entries1),
+    MEs = [ME#menu{name=undefined} || ME <- Entries],
+    MenuData = wx:batch(fun() -> setup_dialog(Parent, MEs, Magnet, Pos, ?GET(menu_cache)) end),
     Env = wx:get_env(),
-    Pid = spawn_link(fun() ->
+    spawn_link(fun() ->
 		       try
 			   wx:set_env(Env),
-			   {Frame, Panel, MEs, Cols} = wx:batch(CreateMenu),
-			   popup_events(Frame, Panel, MEs, Cols, Magnet, undefined, Names, Owner),
-                           wxWindow:releaseMouse(Panel),
-                           wxWindow:hide(Frame),
-                           wxFrame:destroy(Frame)
-		       catch _:Reason ->
-			       io:format("CRASH ~p ~p~n",[Reason, erlang:get_stacktrace()])
+                           register(wings_menu_process, self()),
+                           send_enter_window(MenuData, Pos),
+			   popup_events(MenuData, Magnet, undefined, Names, Owner),
+                           close_menu_frame(MenuData),
+                           wxWindow:setFocus(Parent)
+		       catch _:Reason:ST ->
+			       io:format("CRASH ~p ~p~n",[Reason, ST])
 		       end,
 		       normal
 	       end),
-    {Pid,Entries}.
+    Entries.
 
-setup_dialog(Parent, Entries0, Magnet, {X0,Y0}=ScreenPos) ->
-    X  = X0-20,
-    Y1 = Y0-10,
-    Flags = ?wxFRAME_TOOL_WINDOW bor ?wxFRAME_FLOAT_ON_PARENT bor ?wxFRAME_NO_TASKBAR,
-    Frame = wxFrame:new(),
-    wxWindow:setExtraStyle(Frame, ?wxWS_EX_PROCESS_IDLE bor ?wxFRAME_EX_METAL),
-    %% case {os:type(), {?wxMAJOR_VERSION, ?wxMINOR_VERSION}} of
-    %%     {{_, linux}, Ver} when Ver > {3,0} ->
-    %%         wxFrame:setBackgroundStyle(Frame, 3); %% ?wxBG_STYLE_TRANSPARENT
-    %%     _ -> ok
-    %% end,
-    true = wxFrame:create(Frame, Parent, -1, "", [{style, Flags}]),
-    Panel = wxWindow:new(Frame, -1, [{style, ?wxWANTS_CHARS}]),
+setup_dialog(Parent, Entries, Magnet, ScreenPos, ignore) ->
+    do_setup_dialog(Parent, Entries, Magnet, ScreenPos);
+setup_dialog(Parent, Entries, Magnet, ScreenPos, undefined) ->
+    setup_dialog(Parent, Entries, Magnet, ScreenPos, #{});
+setup_dialog(Parent, Entries, Magnet, ScreenPos, Cache) ->
+    TopParent = get_toplevel(Parent),
+    case maps:get({Entries, TopParent}, Cache, undefined) of
+        undefined ->
+            MenuData = do_setup_dialog(TopParent, Entries, Magnet, ScreenPos),
+            case maps:get(overlay, MenuData) of
+                none ->
+                    %% Note we leak popup windows menus here,
+                    %% like autouv, should it be cleaned up?
+                    ?SET(menu_cache, Cache#{{Entries, TopParent} => MenuData});
+                _ ->  %% Only for popuptransient windows
+                    ?SET(menu_cache,ignore)
+            end,
+            MenuData;
+        #{frame := Frame} = MenuData ->
+            Pos = fit_menu_on_display(Frame,ScreenPos),
+            wxWindow:move(Frame, Pos),
+            wxPopupTransientWindow:popup(Frame),
+            MenuData
+    end.
+
+do_setup_dialog(TopParent, Entries0, Magnet, ScreenPos) ->
+    {Overlay, Frame, KbdFocus} = make_menu_frame(TopParent, ScreenPos),
+    Panel = wxPanel:new(Frame),
     wxWindow:setFont(Panel, ?GET(system_font_wx)),
     {{R,G,B,A},FG} = {colorB(menu_color),colorB(menu_text)},
     Cols = {{R,G,B,A}, FG},
@@ -235,95 +246,187 @@ setup_dialog(Parent, Entries0, Magnet, {X0,Y0}=ScreenPos) ->
     wxPanel:setSizer(Panel, Main),
     wxSizer:fit(Main, Panel),
     wxWindow:setClientSize(Frame, wxWindow:getSize(Panel)),
-    wxWindow:connect(Frame, show),
-    {_, MaxH} = wx_misc:displaySize(),
-    {_,H} = wxWindow:getSize(Frame),
-    Y = if ((Y1+H) > MaxH) -> max(0, (MaxH-H-5));
-           true -> Y1
-        end,
-    [wxWindow:connect(Panel, Ev, [{skip, false}]) ||
-        Ev <- [motion, left_up, middle_up, right_up]],
-    wxWindow:move(Frame, {X,Y}),
-    wxPanel:connect(Panel, char),
-    wxPanel:connect(Panel, char_hook),
-    catch wxWindow:connect(Panel, mouse_capture_lost), %% Not available in old wx's.
-    wxFrame:show(Frame),
+    wxWindow:move(Frame, fit_menu_on_display(Frame, ScreenPos)),
+    show_menu_frame(Overlay, Frame, KbdFocus),
+    #{overlay=>Overlay, frame=>Frame, panel=>Panel, entries=>Entries, colors=>Cols}.
 
-    %% Color active menuitem
-    {MX, MY} = wxWindow:screenToClient(Panel, ScreenPos),
-    MEv = #wxMouse{type=motion,x=MX,y=MY,
-                   leftDown=false,middleDown=false,rightDown=false,
-                   controlDown=false,shiftDown=false,altDown=false,metaDown=false,
-                   wheelRotation=0, wheelDelta=0, linesPerAction=0
-                  },
-    self() ! #wx{id=-1, obj=wx:null(), event=MEv},
-    {Frame, Panel, Entries, Cols}.
-
-popup_events(Frame, Panel, Entries, Cols, Magnet, Previous, Ns, Owner) ->
-    receive
-	#wx{event=#wxMouse{x=X,y=Y,type=motion}} ->
-            Do = fun() ->
-                         case find_active_panel(Panel, X, Y) of
-                             {Id, Obj} when is_number(Id), Obj =/= Previous ->
-                                 wings_status:message(Owner, entry_msg(Id, Entries), ""),
-                                 {BG,FG} = Cols,
-                                 setup_colors(Previous, BG, FG),
-                                 setup_colors(Obj, colorB(menu_hilite),colorB(menu_hilited_text)),
-                                 Obj;
-                             _ ->
-                                 Previous
-                         end
-                 end,
-            Line = wx:batch(Do),
-            popup_events(Frame, Panel, Entries, Cols, Magnet, Line, Ns, Owner);
-	#wx{event=Ev=#wxMouse{y=Y, x=X}} ->
-	    What = mouse_button(Ev),
-	    case find_active_panel(Panel, X, Y) of
-		{false, outside} when What =:= right_up ->
-		    Pos = wxWindow:clientToScreen(Frame,{X,Y}),
-                    wxWindow:move(Frame, Pos),
-                    wings_wm:psend(Owner, redraw),
-		    popup_events(Frame, Panel, Entries, Cols, Magnet, Previous, Ns, Owner);
-		{false, outside} ->
-		    wings_wm:psend(Owner, cancel);
-                {false, inside} ->
-                    popup_events(Frame, Panel, Entries, Cols, Magnet, Previous, Ns, Owner);
-                {PanelId, RowPanel} ->
-                    {PX,_} = wxWindow:getPosition(RowPanel),
-                    Id = get_hit_id(X-PX, RowPanel, PanelId),
-		    MagnetClick = Magnet orelse
-			magnet_pressed(wings_msg:free_rmb_modifier(), Ev),
-		    wings_wm:psend(Owner, {click, Id, {What, MagnetClick}, Ns})
-	    end;
-	#wx{event=#wxShow{show=true}} ->
-            capture_mouse(Frame, Panel),
-            popup_events(Frame, Panel, Entries, Cols, Magnet, Previous, Ns, Owner);
-	#wx{event=#wxKey{keyCode=Key}} = _Ev ->
-	    if Key =:= ?WXK_ESCAPE ->
-		    wings_wm:psend(Owner, cancel);
-	       true ->
-		    popup_events(Frame, Panel, Entries, Cols, Magnet, Previous, Ns, Owner)
-	    end;
-        cancel ->
-            wings_wm:psend(Owner, cancel);
-        #wx{event=#wxMouseCaptureLost{}} ->
-            wings_wm:psend(Owner, cancel);
-	_Ev ->
-	    ?dbg("Got Ev ~p ~n", [_Ev]),
-	    popup_events(Frame, Panel, Entries, Cols, Magnet, Previous, Ns, Owner)
+get_toplevel(Win) ->
+    Parent = try wxWindow:isTopLevel(Win) of
+                 true  -> wx:null();
+                 false -> wxWindow:getParent(Win)
+             catch _:_ ->
+                     no_exists
+             end,
+    case wx:is_null(Parent) of
+        true -> Win;
+        false -> get_toplevel(Parent)
     end.
 
-capture_mouse(Frame, Panel) ->
-    wxWindow:disconnect(Frame, show),
-    case os:type() of          %% Capture mouse on GTK must be done on an realized window
-        {unix, linux} -> timer:sleep(150); %% sleep so we ensure that (async) creation is done
-        _ -> ok
-    end,
-    wxWindow:setFocus(Panel),
-    wxWindow:captureMouse(Panel),
-    ok.
+show_menu_frame(none, Frame, _Focus) ->
+    wxPopupTransientWindow:popup(Frame);
+show_menu_frame(_, Frame, KbdFocus) ->
+    KbdFocus(),  %% Set keyboard focus so we can catch ESC on mac
+    wxFrame:show(Frame).
 
-%% If the mouse is not moved after popping up the menu, the meny entry
+close_menu_frame(#{overlay:=none, frame:=Frame}) ->
+    wxPopupTransientWindow:dismiss(Frame);
+close_menu_frame(#{overlay:=Overlay, frame:=Frame}) ->
+    wxWindow:hide(Frame),
+    wxFrame:destroy(Overlay).
+
+send_enter_window(#{panel:=Panel}, ScreenPos) ->
+    %% Color active menuitem
+    {MX, MY} = wxWindow:screenToClient(Panel, ScreenPos),
+    case find_active_panel(Panel, MX, MY) of
+	{false,_} -> ignore;
+	{ActId, ActPanel} ->
+	    self() ! #wx{id=ActId, obj=ActPanel,
+			 event=#wxMouse{type=enter_window,x=0,y=0,
+					leftDown=false,middleDown=false,rightDown=false,
+					controlDown=false,shiftDown=false,altDown=false,metaDown=false,
+					wheelRotation=0, wheelDelta=0, linesPerAction=0}}
+    end.
+
+make_menu_frame(Parent, Pos) ->
+    case os:type() of
+        {_, darwin} ->
+            %% PopupTransientWindow did not work on Mac on wxWidgets 3.1.3 atleast
+            %% So we make our on own overlay handling there
+            %% on the other transparent windows don't work on some linux'es
+            %% So we can't use this for all OS's
+            make_overlay(Parent,Pos);
+        _ ->
+            Frame = wxPopupTransientWindow:new(Parent, [{style, ?wxBORDER_SIMPLE}]),
+            EvH = fun(Ev, _) -> catch wings_menu_process ! Ev end,
+            wxPopupTransientWindow:connect(Frame, show, [{callback, EvH}]),
+            {none, Frame, none}
+    end.
+
+make_overlay(Parent, ScreenPos) ->
+    OL = wxFrame:new(),
+    Flags = ?wxFRAME_TOOL_WINDOW bor ?wxFRAME_FLOAT_ON_PARENT bor ?wxFRAME_NO_TASKBAR,
+    TCol = case {os:type(), {?wxMAJOR_VERSION, ?wxMINOR_VERSION}} of
+               {{_, linux}, Ver} when Ver >= {3,0} ->
+                   wxFrame:setBackgroundStyle(OL, 3), %% ?wxBG_STYLE_TRANSPARENT
+                   0;
+               {{_, darwin}, _} ->
+                   13;  %% No events received if completely transparent ??
+               _ ->
+                   1
+           end,
+    DisplayID = wxDisplay:getFromPoint(ScreenPos),
+    Display = wxDisplay_new(DisplayID),
+    {DX,DY,DW,DH} = wxDisplay:getClientArea(Display),
+    true = wxFrame:create(OL, Parent, -1, "", [{pos,{DX,DY}},{size, {DW,DH}},{style, Flags}]),
+    wxFrame:setBackgroundColour(OL, {0,0,0,TCol}),
+    catch wxFrame:setTransparent(OL, TCol),
+    Panel = wxWindow:new(OL, -1, [{size, {DW,DH}}, {style, ?wxWANTS_CHARS}]),
+    EvH = fun(#wx{event=#wxKey{keyCode=Key}}, _) ->
+                  if Key =:= ?WXK_ESCAPE -> wings_menu_process ! cancel;
+                     true -> ok
+                  end;
+             (#wx{event=#wxMouse{type=right_up,y=Y, x=X}}, _) ->
+                  wings_menu_process ! {move, wxWindow:clientToScreen(OL,{X,Y})};
+             (_Ev, Obj) ->
+                  wxEvent:skip(Obj),
+                  %% ?dbg("Cancel menu: ~w~n",[_Ev]),
+                  catch wings_menu_process ! cancel
+          end,
+    [wxWindow:connect(Panel, Ev, [{callback, EvH}]) ||
+        Ev <- [left_up, middle_up, right_up, char, char_hook]],
+    wxFrame:show(OL),
+
+    FrameFlags = case os:type() of
+                     {_, linux} -> ?wxSTAY_ON_TOP;  %% Hmm needed for some reason
+                     _ -> ?wxFRAME_FLOAT_ON_PARENT
+                 end,
+    Flags = ?wxFRAME_TOOL_WINDOW bor FrameFlags bor ?wxFRAME_NO_TASKBAR,
+    Frame = wxFrame:new(OL, -1, "", [{style, FrameFlags}]),
+
+    {OL, Frame, fun() -> wxWindow:setFocus(Panel) end}.
+
+wxDisplay_new(DisplayID) ->
+    New = wings_u:id(new),
+    try  %% new
+        wxDisplay:New(DisplayID)
+    catch _:_ -> %% old
+            wxDisplay:New([{n, DisplayID}])
+    end.
+
+popup_events(MenuData, Magnet, Previous, Ns, Owner) ->
+    receive
+	#wx{id=Id, obj=Obj,event=#wxMouse{type=enter_window}} ->
+            Set = fun() ->
+                          if Obj =/= Previous ->
+                                  {BG,FG} = maps:get(colors, MenuData),
+                                  setup_colors(Previous, BG, FG),
+                                  setup_colors(Obj, colorB(menu_hilite),colorB(menu_hilited_text)),
+                                  Obj;
+                             true ->
+                                  Previous
+                          end
+                  end,
+	    Line = wx:batch(Set),
+	    wings_status:message(Owner, entry_msg(Id, maps:get(entries, MenuData)), ""),
+            popup_events(MenuData, Magnet, Line, Ns, Owner);
+	#wx{id=Id0, event=Ev=#wxMouse{y=Y, x=X}} ->
+            Id = case Id0 > 0 orelse find_active_panel(maps:get(panel, MenuData), X, Y) of
+		     true -> Id0;
+		     {false, _} = No -> No;
+		     {AId, _} -> AId
+		 end,
+            %% ?dbg("Ev: ~w ~w ~w~n",[Ev, Id0, Id]),
+	    case Id of
+		{false, outside} ->
+                    {BG,FG} = maps:get(colors, MenuData),
+                    setup_colors(Previous, BG, FG),
+		    wings_wm:psend(Owner, cancel);
+                {false, inside} ->
+                    popup_events(MenuData, Magnet, Previous, Ns, Owner);
+                Active when is_integer(Active) ->
+                    {BG,FG} = maps:get(colors, MenuData),
+                    setup_colors(Previous, BG, FG),
+		    MagnetClick = Magnet orelse
+			magnet_pressed(wings_msg:free_rmb_modifier(), Ev),
+		    wings_wm:psend(Owner, {click, Id, {mouse_button(Ev), MagnetClick}, Ns})
+	    end;
+        cancel ->
+            {BG,FG} = maps:get(colors, MenuData),
+            setup_colors(Previous, BG, FG),
+            wings_wm:psend(Owner, cancel);
+        {move, {X,Y}} ->
+            Frame = maps:get(frame, MenuData),
+            Pos = fit_menu_on_display(Frame, {X,Y}),
+            wxWindow:move(Frame, Pos),
+            wings_wm:psend(Owner, redraw),
+            popup_events(MenuData, Magnet, Previous, Ns, Owner);
+        #wx{event=#wxShow{show=false}} ->
+            {BG,FG} = maps:get(colors, MenuData),
+            setup_colors(Previous, BG, FG),
+            wings_wm:psend(Owner, cancel);
+	_Ev ->
+	    %% ?dbg("Got Ev ~p ~n", [_Ev]),
+	    popup_events(MenuData, Magnet, Previous, Ns, Owner)
+    end.
+
+fit_menu_on_display(Frame, {MX,MY} = Pos) ->
+    {WW,WH} = wxWindow:getSize(Frame),
+    DisplayID = wxDisplay:getFromPoint(Pos),
+    Display = wxDisplay_new(DisplayID),
+    {DX,DY,DW,DH} = wxDisplay:getClientArea(Display),
+    MaxW = (DX+DW),
+    PX = if (MX+WW) > MaxW -> MaxW-WW-2;
+            (MX-25) < DX -> MX;
+            true -> max(5, (MX-25)) %% Move so mouse is inside menu
+         end,
+    MaxH = (DY+DH),
+    PY = if (MY+WH) > MaxH -> MaxH-WH-2;
+            true -> max(5, (MY-15)) %% Move so mouse is inside menu
+         end,
+    wxDisplay:destroy(Display),
+    {PX,PY}.
+
+%% If the mouse is not moved after popping up the menu, the menu entry
 %% is not active, find_active_panel finds the active row.
 find_active_panel(Panel, MX, MY) ->
     {_,_,WinW,WinH} = wxWindow:getRect(Panel),
@@ -356,27 +459,6 @@ find_active_panel_1(Panel, MY) ->
 	    Found
     end.
 
-get_hit_id(MX, Panel, Id0) ->
-    Sizer = wxWindow:getSizer(Panel),
-    Children = wxSizer:getChildren(Sizer),
-    Check = fun(Item) ->
-                    {X, _, W, _} = wxSizerItem:getRect(Item),
-                    if MX < X -> false;
-                       MX >= (X+W) -> false;
-                       true ->
-                            Active = wxSizerItem:getWindow(Item),
-                            case wx:is_null(Active) orelse wxWindow:getId(Active) of
-                                true -> false;
-                                Id when Id < 0 -> false;
-                                Id -> {true, Id}
-                            end
-                    end
-            end,
-    case lists:filtermap(Check, Children) of
-        [] -> Id0;
-        [Id|_] -> Id
-    end.
-
 magnet_pressed(?CTRL_BITS, #wxMouse{controlDown=true}) -> true;
 magnet_pressed(?ALT_BITS, #wxMouse{altDown=true}) -> true;
 magnet_pressed(?META_BITS, #wxMouse{metaDown=true}) -> true;
@@ -403,31 +485,24 @@ mouse_button(#wxMouse{type=What, controlDown = Ctrl, altDown = Alt, metaDown = M
     end.
 
 popup_event_handler(cancel, _, _) ->
-    mac_fix() andalso wings_wm:release_focus(),
-    pop;
-popup_event_handler({click, Id, Click, Ns}, {Parent,Owner,_Pid}, Entries0) ->
+    wings_wm:release_focus(),
+    delete;
+popup_event_handler({click, Id, Click, Ns}, {Parent,Owner}, Entries0) ->
     case popup_result(lists:keyfind(Id, 2, Entries0), Click, Ns, Owner) of
 	pop ->
-            mac_fix() andalso wings_wm:release_focus(),
-            pop;
+            wings_wm:release_focus(),
+            delete;
 	{submenu, Names, Menus, MagnetClick} ->
-	    {_, X0, Y0} = wings_io:get_mouse_state(),
-	    Pos = wxWindow:screenToClient(wings_wm:this_win(), {X0,Y0}),
-	    {X,Y} = wxWindow:clientToScreen(wings_wm:this_win(), Pos),
-	    {Pid,Entries} = wx_popup_menu(Parent, {X,Y}, Names, Menus, MagnetClick, Owner),
-	    {replace, fun(Ev) -> popup_event_handler(Ev, {Parent,Owner,Pid}, Entries) end}
+	    {_, X,Y} = wings_io:get_mouse_state(),
+	    Entries = wx_popup_menu(Parent, {X,Y}, Names, Menus, MagnetClick, dialog_blanket),
+	    {replace, fun(Ev) -> popup_event_handler(Ev, {Parent,Owner}, Entries) end}
     end;
 popup_event_handler(redraw,_,_) ->
-    defer;
-popup_event_handler(#mousemotion{},_,_) ->
     keep;
-popup_event_handler(#keyboard{sym=?SDLK_ESCAPE}, {_, _, Pid}, _) ->
-    Pid ! cancel, %% Keyboard focus fails on mac wxWidgets-3.1.3
+popup_event_handler(#keyboard{sym=?SDLK_ESCAPE}, {_, _}, _) ->
+    wings_menu_process ! cancel, %% Keyboard focus fails on mac wxWidgets-3.1.3
     keep;
-popup_event_handler(lost_focus, {_, _, Pid}, _) ->
-    %% just focus lost
-    %% mac wxWidgets capture mouse outside window does not get mouse click
-    mac_fix() andalso (Pid ! cancel),
+popup_event_handler(#mousemotion{}, _, _) ->
     keep;
 popup_event_handler(_Ev,_,_) ->
     %% io:format("Hmm ~p ~n",[_Ev]),
@@ -529,6 +604,7 @@ setup_popup([#menu{type=submenu, wxid=Id, desc=Desc, help=Help0, opts=Ps, hk=HK}
     [wxWindow:setToolTip(Win, wxToolTip:new(TipMsg)) || Win <- Controls],
     wxPanel:setSizerAndFit(Panel, Line),
     wxSizer:add(Sizer, Panel, [{flag, ?wxEXPAND},{proportion, 1}]),
+    menu_connect(Controls, [left_up, middle_up, right_up, enter_window]),
     setup_popup(Es, Sizer, Sz, Cs, Parent, Magnet, [ME#menu{help=CmdMsg}|Acc]);
 setup_popup([#menu{type=menu, wxid=Id, desc=Desc, help=Help, opts=Props, hk=HK}=ME|Es],
 	    Sizer, Sz = {Sz1,Sz2}, Cs, Parent, Magnet, Acc) ->
@@ -548,7 +624,13 @@ setup_popup([#menu{type=menu, wxid=Id, desc=Desc, help=Help, opts=Props, hk=HK}=
     BM = case {OpBox = have_option_box(Props),have_color(Props)} of
 	     {true,_} ->
 		 Bitmap = get_pref_bitmap(),
-		 SBM = wxStaticBitmap:new(Panel, Id+1, Bitmap),
+                 SBM = case os:type() of
+                           {_, darwin} ->
+                               wxBitmapButton:new(Panel, Id+1, Bitmap,
+                                                  [{style,?wxNO_BORDER}]);
+                           _ ->
+                               wxStaticBitmap:new(Panel, Id+1, Bitmap)
+                       end,
 		 wxSizer:add(Line, SBM, [{flag, ?wxALIGN_CENTER}]),
 		 [SBM];
 	     {false, true} ->
@@ -567,6 +649,7 @@ setup_popup([#menu{type=menu, wxid=Id, desc=Desc, help=Help, opts=Props, hk=HK}=
     [wxWindow:setToolTip(Win, wxToolTip:new(TipMsg)) || Win <- [Panel,T1,T2|BM]],
     wxPanel:setSizerAndFit(Panel, Line),
     wxSizer:add(Sizer, Panel, [{flag, ?wxEXPAND}, {proportion, 1}]),
+    menu_connect([Panel,T1,T2|BM], [left_up, middle_up, right_up, enter_window]),
     Win = #{panel=>Panel, label=>T1, hotkey=>T2},
     Pop = ME#menu{wxid=Id, help=CmdMsg, object=Win},
     setup_popup(Es, Sizer, Sz, Cs, Parent, Magnet, [Pop|Acc]);
@@ -574,6 +657,10 @@ setup_popup([#menu{type=opt}=ME|Es], Sizer, Sz, Cs, Parent, Magnet, Acc) ->
     setup_popup(Es, Sizer, Sz, Cs, Parent, Magnet, [ME|Acc]);
 setup_popup([], _, _, _, _, _, Acc) -> lists:reverse(Acc).
 
+menu_connect(Windows, Evs) ->
+    EvH = fun(Ev, _) -> wings_menu_process ! Ev end,
+    [ [wxWindow:connect(Win, Ev, [{callback, EvH}]) || Ev <- Evs] || Win <- Windows].
+
 get_pref_bitmap() ->
     case ?GET(small_pref_bm) of
 	undefined ->
@@ -898,15 +985,18 @@ id_to_name(Id) ->
     [#menu{name=Name}] = ets:lookup(wings_menus, Id),
     Name.
 
-setup_menu(Names, Id, Menus1) when is_list(Menus1) ->
+setup_menu(Names, Id, Menus) ->
     Menu   = wxMenu:new(),
-    Menus2  = wings_plugin:menu(list_to_tuple(reverse(Names)), Menus1),
-    HotKeys = wings_hotkey:matching(Names),
-    Menus3 = [normalize_menu_wx(Entry, HotKeys, Names) || Entry <- Menus2],
-    Menus = format_hotkeys(Menus3, wx),
-    Next  = create_menu(Menus, Id, Names, Menu),
+    Entries = make_entries(Names, Menus, wx),
+    Next  = create_menu(Entries, Id, Names, Menu),
     {Menu, Next}.
 
+make_entries(Names, Menus0, Style) when is_list(Menus0) ->
+    Menus1  = wings_plugin:menu(list_to_tuple(reverse(Names)), Menus0),
+    HotKeys = wings_hotkey:matching(Names),
+    Menus2 = [normalize_menu_wx(Entry, HotKeys, Names) || Entry <- lists:flatten(Menus1)],
+    format_hotkeys(Menus2, Style).
+
 normalize_menu_wx(separator, _, _) ->
     #menu{type=separator};
 normalize_menu_wx({S,Fun,Help,Ps}, Hotkeys, Ns) when is_function(Fun) ->
