@@ -195,11 +195,8 @@ scan_color(face, Faces, We) ->
 scan_materials([Mat|Ms], Cols) ->
     Opengl = proplists:get_value(opengl, Mat),
     Diff   = proplists:get_value(diffuse, Opengl),
-    Amb    = proplists:get_value(ambient, Opengl),
-    Spec   = proplists:get_value(specular, Opengl),
     Emis   = proplists:get_value(emission, Opengl),
-    scan_materials(Ms, [color(Diff),color(Amb),color(Spec),
-			color(Emis)|Cols]);
+    scan_materials(Ms, [color(Diff),color(Emis)|Cols]);
 scan_materials([], Cols) -> Cols.
 
 add_cols([none|R], Acc) ->
@@ -263,8 +260,10 @@ init([Frame, {W,_}, _Ps, Cols0]) ->
 	wxWindow:destroy(Tmp),
 	{ColsW,ColsH} = calc_size(Cols0,W,BW,false),
 	%% io:format("Init Size ~p => ~p~n",[BSz, {ColsW, ColsH}]),
-	Win = wxScrolledWindow:new(Frame),
-	Sz = wxGridSizer:new(ColsW, [{vgap, ?BORD},{hgap, ?BORD}]),
+	Win = wxScrolledWindow:new(Frame, [{style, wings_frame:get_border()}]),
+        #{bg:=BG} = wings_frame:get_colors(),
+        wxPanel:setBackgroundColour(Win, BG),
+	Sz = wxGridSizer_new(ColsW, ?BORD),
 	Cols = add_empty(Cols0,ColsW,ColsH),
 	manage_bitmaps(Win, Sz, Cols, [], Empty),
 	BorderSz = wxBoxSizer:new(?wxHORIZONTAL),
@@ -278,11 +277,20 @@ init([Frame, {W,_}, _Ps, Cols0]) ->
 	wings_status:message(palette, help()),
 	{Win, #state{self=self(), win=Win, sz=Sz,
 		     cols=Cols, bsz=BSz, empty=Empty}}
-    catch _:Reason ->
-	    io:format("CRASH: ~p ~p ~p~n",[?MODULE, Reason, erlang:get_stacktrace()]),
+    catch _:Reason:ST ->
+	    io:format("CRASH: ~p ~p ~p~n",[?MODULE, Reason, ST]),
             error(Reason)
     end.
 
+wxGridSizer_new(ColsW, Size) ->
+    Func = wings_u:id(new),
+    case wings_u:is_exported(wxGridSizer, new, 3) of
+        true ->
+            wxGridSizer:Func(ColsW, Size, Size);
+        false ->
+            wxGridSizer:Func(ColsW, [{vgap, Size},{hgap, Size}])
+    end.
+
 %%%%%%%%%%%%%%%%%%%%%%%
 handle_event(#wx{event=#wxMouse{type=enter_window}}=Ev, State) ->
     wings_frame ! Ev,
@@ -526,9 +534,13 @@ make_button(Parent, Id, FloatCol, Empty) ->
 		       none -> {false, Empty};
 		       _ -> {true, make_bitmap(FloatCol)}
 		   end,
-    Static = case os:type() of
-		 {win32, _} -> wxBitmapButton:new(Parent, Id, BM, [{style, ?wxBORDER_NONE}]);
-		 {_, _}     -> wxStaticBitmap:new(Parent, Id, BM)
+    Static = case {os:type(), {?wxMAJOR_VERSION, ?wxMINOR_VERSION}} of
+		 {{win32, _},_} ->
+                     wxBitmapButton:new(Parent, Id, BM, [{style, ?wxBORDER_NONE}]);
+                 {{_,darwin},Ver} when Ver > {3,0} ->
+                     wxBitmapButton:new(Parent, Id, BM, [{style, ?wxBORDER_NONE}]);
+		 {_, _}     ->
+                     wxStaticBitmap:new(Parent, Id, BM)
 	     end,
     Delete andalso wxBitmap:destroy(BM),
     [wxWindow:connect(Static, Ev, [{skip, false}]) ||
@@ -557,9 +569,13 @@ setColor(Id, Color, _, Parent) ->
 setBitmap(Id, Bitmap, Parent) ->
     Win = wxWindow:findWindow(Parent, Id),
     wx:is_null(Win) andalso error({no_such_id, Id}),
-    case os:type() of
-	{win32, _} -> wxBitmapButton:setBitmapLabel(wx:typeCast(Win, wxBitmapButton), Bitmap);
-	{_, _}     -> wxStaticBitmap:setBitmap(wx:typeCast(Win, wxStaticBitmap), Bitmap)
+    case  {os:type(), {?wxMAJOR_VERSION, ?wxMINOR_VERSION}} of
+	{{win32, _},_} ->
+            wxBitmapButton:setBitmapLabel(wx:typeCast(Win, wxBitmapButton), Bitmap);
+        {{_,darwin}, Ver} when Ver > {3,0} ->
+            wxBitmapButton:setBitmapLabel(wx:typeCast(Win, wxBitmapButton), Bitmap);
+	{_, _}     ->
+            wxStaticBitmap:setBitmap(wx:typeCast(Win, wxStaticBitmap), Bitmap)
     end.
 
 make_bitmap(Col0) ->
