@@ -187,20 +187,15 @@ handle_hilite_event(#mousemotion{x=X,y=Y}, #hl{prev=PrevHit}=HL0) ->
 	    wings_wm:dirty(),
 	    get_hilite_event(HL0);
 	PrevHit ->
-	    case tweak_vector() of
-	      true ->
-	        case tweak_hilite(X,Y,St) of
-	          none ->
-	            HL = HL0;
-	          Hit ->
-	            wings_wm:dirty(),
-	            insert_tweak_vector(PrevHit, Hit, St),
-	            HL = HL0#hl{prev={PrevHit,Hit}}
-	        end;
-	      _ ->
-	        HL = HL0
-	    end,
-	    wings_draw:refresh_dlists(St),
+	    HL = case tweak_vector() andalso tweak_hilite(X,Y,St) of
+                     false -> HL0;
+                     none ->  HL0;
+                     Hit ->
+                         wings_wm:dirty(),
+                         insert_tweak_vector(PrevHit, Hit, St),
+                         wings_draw:refresh_dlists(St),
+                         HL0#hl{prev={PrevHit,Hit}}
+                 end,
 	    get_hilite_event(HL);
 	none ->
 	    wings_wm:dirty(),
@@ -344,7 +339,7 @@ hilite_draw_sel_fun(vertex, V, #dlo{src_we=#we{vp=Vtab}}) ->
     wings_vbo:new(D, Data);
 hilite_draw_sel_fun(edge, Edge, #dlo{src_we=#we{es=Etab,vp=Vtab}}) ->
     #edge{vs=Va,ve=Vb} = array:get(Edge, Etab),
-    LineWidth = wings_pref:get_value(selected_edge_width),
+    LineWidth = float(wings_pref:get_value(selected_edge_width)),
     Data = [array:get(Va, Vtab),array:get(Vb, Vtab)],
     D = fun(RS) ->
 		gl:lineWidth(LineWidth),
@@ -573,8 +568,8 @@ is_all_inside_rect([P|Ps], Rect) ->
     is_inside_rect(P, Rect) andalso is_all_inside_rect(Ps, Rect);
 is_all_inside_rect([], _Rect) -> true.
 
-is_inside_rect({Px,Py,Pz}, {MM,PM,ViewPort,X1,Y1,X2,Y2}) ->
-    {Sx,Sy,_} = wings_gl:project(Px, Py, Pz, MM, PM, ViewPort),
+is_inside_rect(Pos, {MM,PM,ViewPort,X1,Y1,X2,Y2}) ->
+    {Sx,Sy,_} = e3d_transform:project(Pos, MM, PM, ViewPort),
     X1 < Sx andalso Sx < X2 andalso
 	Y1 < Sy andalso Sy < Y2.
 
@@ -827,9 +822,8 @@ find_edge(Face, We, Cx, Cy, Trans) ->
       end, [], Face, We).
 
 project_vertex(V, We, {ModelMatrix,ProjMatrix,ViewPort}) ->
-    {Px,Py,Pz} = wings_vertex:pos(V, We),
-    {Xs,Ys,_}  = wings_gl:project(Px, Py, Pz, ModelMatrix,
-				  ProjMatrix, ViewPort),
+    Pos = wings_vertex:pos(V, We),
+    {Xs,Ys,_}  = e3d_transform:project(Pos, ModelMatrix, ProjMatrix, ViewPort),
     {Xs,Ys}.
 
 check_restriction({Mode,MM,_}=Hilite, Id, V, Edge, Face) ->
