@@ -150,6 +150,9 @@ menu() ->
       ?__(70,"Hide all (partly or wholly) selected objects")},
      {?__(71,"Hide Unselected")++Objects,
       hide_unselected,?__(72,"Hide objects that have no selection")},
+     {?__(110,"Lock Selected")++Objects,
+      lock_selected,
+      ?__(111,"Lock all (partly or wholly) selected objects")},
      {?__(73,"Lock Unselected")++Objects,
       lock_unselected,?__(74,"Lock objects that have no selection")},
      separator,
@@ -255,6 +258,8 @@ command(hide_selected, St) ->
     {save_state,hide_selected(St)};
 command(hide_unselected, St) ->
     {save_state,hide_unselected(St)};
+command(lock_selected, St) ->
+    {save_state,lock_selected(St)};
 command(lock_unselected, St) ->
     {save_state,lock_unselected(St)};
 command(show_all, St0) ->
@@ -453,6 +458,10 @@ hide_unselected(St) ->
     Unselected = wings_sel:unselected_ids(St),
     wings_obj:hide(Unselected, St).
 
+lock_selected(St) ->
+    Selected = wings_sel:selected_ids(St),
+    wings_obj:lock(Selected, St).
+
 lock_unselected(St) ->
     Unselected = wings_sel:unselected_ids(St),
     wings_obj:lock(Unselected, St).
@@ -535,7 +544,7 @@ select_group({Mode,_}=Key, #st{ssels=Ssels}=St) ->
     St#st{selmode=Mode,sel=ValidSel}.
 
 %%%% Delete Groups that return an empty selection. Invalid ssels can result from
-%%%% creating or deleting geomerty.
+%%%% creating or deleting geometry.
 delete_invalid_groups(#st{ssels=Ssels}=St0) ->
     case gb_trees:is_empty(Ssels) of
       true ->
@@ -592,7 +601,7 @@ save_group(Key, Sel, #st{ssels=Ssels0}=St) ->
 
 new_group(_) ->
     wings_dialog:ask(?__(1,"Create New Group"),
-		     [{?__(2,"Group Name"), ""}],
+		     [{?__(2,"Group Name"), "",[{width,22}]}],
 		     fun([String]) -> {select,{new_group_name,String}} end).
 
 new_group_name(Name, #st{ssels=Ssels0,selmode=Mode,sel=Sel}=St) ->
@@ -693,7 +702,7 @@ similar(#st{selmode=body}=St) ->
 			   member(Template, Templates)
 		   end, body, St).
 
-do_similar(MakeTemplate, #st{selmode=Mode}=St) ->
+do_similar(MakeTemplate, #st{selmode=Mode,sel=Sel0}=St) when Sel0 =/= [] ->
     MF = fun(Sel, We) ->
 		 Ts = gb_sets:fold(
 			fun(I, A) ->
@@ -708,7 +717,8 @@ do_similar(MakeTemplate, #st{selmode=Mode}=St) ->
       fun(Item, We) ->
 	      Template = MakeTemplate(Item, We),
 	      match_templates(Template, Templates)
-      end, Mode, St).
+      end, Mode, St);
+do_similar(_, St) -> St.
 
 consolidate_templates([H|T]) ->
     consolidate_templates_1(H, T).
